//
// This file is part of the Marble Virtual Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2007   Andrew Manson   <g.real.ate@gmail.com>
// Copyright 2009   Eckhart Wörner  <ewoerner@kde.org>
// Copyright 2010   Thibaut Gridel  <tgridel@free.fr>
//

#ifndef POSITION_MARKER_H
#define POSITION_MARKER_H

#include <QtCore/QObject>
#include <QtCore/QHash>
#include <QtCore/QVector>
#include <QtGui/QColor>
#include <QtGui/QAbstractButton>

#include "DialogConfigurationInterface.h"
#include "RenderPlugin.h"
#include "GeoDataCoordinates.h"
#include "GeoDataLatLonAltBox.h"

namespace Ui
{
    class PositionMarkerConfigWidget;
}

namespace Marble
{
	class MarbleWidget;
	class GeoDataLineString;
	class GeoDataPolygon;

class PositionMarker  : public RenderPlugin, public DialogConfigurationInterface
{
    Q_OBJECT
    Q_INTERFACES( Marble::RenderPluginInterface )
	Q_INTERFACES( Marble::DialogConfigurationInterface )
    MARBLE_PLUGIN( PositionMarker )
 public:
    PositionMarker ();
	explicit PositionMarker( const MarbleModel *marbleModel );
    ~PositionMarker ();

    QStringList renderPosition() const;

    QString renderPolicy() const;

    QStringList backendTypes() const;

    QString name() const;

    QString guiString() const;

    QString nameId() const;

	QString version() const;

    QString description() const;

	QString copyrightYears() const;

    QList<PluginAuthor> pluginAuthors() const;

    QIcon icon () const;

    QDialog *configDialog();

    void initialize ();

    bool isInitialized () const;

    bool render( GeoPainter *painter, ViewportParams *viewport,
                 const QString& renderPos, GeoSceneLayer * layer = 0 );


    // Overriding LayerInterface to paint on top of the route
    virtual qreal zValue() const;

    /**
     * @return: The settings of the item.
     */
    virtual QHash<QString,QVariant> settings() const;

    /**
     * Set the settings of the item.
     */
    virtual void setSettings( const QHash<QString,QVariant> &settings );


 public slots:
    void readSettings();
    void writeSettings();
    void setPosition( const GeoDataCoordinates &position );
    void chooseCustomCursor();
    void chooseColor();
    void resizeCursor( int step );

	//enz
	void positionProviderDisabled(bool);

Q_SIGNALS:
	void signalAddObject(QStringList qsl,long id,int type,long& objectID);
	void signalAddImageObject(QString&,long id,int type,long xoffs, long yoffs, long align, long idx, long& objectID,bool multi);
	void signalAddRect(long id,int type,double left, double top,double right,double bottom, long& objectID);
	void signalAddPoly(long id,int type,long lCol,GeoDataLineString&, long& objectID);
	void signalAddPolygon(long id,int type,long lCol,GeoDataLineString&, long& objectID);
	void signalAddPlotObject(QStringList&,long id,int type,double radius,double longitude,double latitude,double minos,double major, long flags, long& objectID);
	void signalAddEllipse(QString&,long id,int type,double radius,double longitude,double latitude, double minor, double major, long& objectID);
	void signalSetObjectProperties(QString& qs,long id,int type);
	void signalAddDescription(QString&,long id,long align,long xoff,long yoff, long& objectID);
	void signalAddAzimuthLine(long id,int type,long azimuth,double longitude,double latitude,GeoDataLineString&,long& objectID,bool multi);

	void signalUpdatePoly(long id,int type,long lCol,GeoDataLineString&, long& objectID);

protected:
    bool eventFilter(QObject* watched, QEvent* e);

private:
	void calcDistanceAzimuth();

 private:
    Q_DISABLE_COPY( PositionMarker )

	void update( const ViewportParams *viewport);//, MapQuality mapQuality );
    void loadCustomCursor( const QString& filename, bool useCursor );
    void loadDefaultCursor();

    bool           m_isInitialized;
    bool           m_useCustomCursor;
    
    const QString m_defaultCursorPath;
    GeoDataLatLonAltBox m_lastBoundingBox;
    GeoDataCoordinates  m_currentPosition;
    GeoDataCoordinates  m_previousPosition;
    
    Ui::PositionMarkerConfigWidget *ui_configWidget;
    QDialog *m_configDialog;
    QString m_cursorPath;

    QPolygonF           m_arrow;
    QPolygonF           m_previousArrow;
    QRegion             m_dirtyRegion;
    QPixmap             m_customCursor;
	QPixmap             m_customCursorTransformed;
    QPixmap             m_defaultCursor;
    float               m_cursorSize;
    QColor              m_accuracyColor;
    QColor              m_trailColor;
    qreal               m_heading;

    QVector<GeoDataCoordinates> m_trail;
    static const int    sm_numTrailPoints = 6;
    bool                m_showTrail;

	//for setting placemarks
	qreal               m_prevHeading;
	qreal				m_prevLon;
	qreal				m_prevLat;
	qreal				m_prevSpeed;
	qreal				m_prevDistance;

    static const int sm_defaultSizeStep;
    static const int sm_numResizeSteps;
    static const float sm_resizeSteps[];

	MarbleWidget*		m_marbleWidget;
	GeoDataLineString*	m_gpsTrack;
	long				m_lOverlay;
	long				m_lLayer;
	long				m_lGroup;
	long				m_lPoly;

private:
    
	//add overlay
	void addOverlay(void);
	//add layer
	void addLayer(void);
	//add group
	long addGroup(qreal,qreal);
	//add track
	long addTrack(const GeoDataCoordinates& pos);
};

}

#endif
