//
// This file is part of the Marble Virtual Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2011 Thibaut Gridel <tgridel@free.fr>

#include "KmlRunner.h"

#include "GeoDataDocument.h"
#include "KmlParser.h"

#include "GeoDataStyle.h"
#include "GeoDataCoordinates.h" 
#include "GeoDataParser.h"
#include "GeoDataTypes.h"
#include "GeoDataData.h"
#include "GeoDataFolder.h"
#include "GeoDataPlacemark.h"
#include "GeoDataLineString.h" 
#include "GeoDataLinearRing.h"
#include "GeoDataIconStyle.h"
#include "GeoDataExtendedData.h"

#include <QtCore/QFile>

namespace Marble
{

KmlRunner::KmlRunner(QObject *parent) :
    MarbleAbstractRunner(parent)
{
}

KmlRunner::~KmlRunner()
{
}

GeoDataFeature::GeoDataVisualCategory KmlRunner::category() const
{
    return GeoDataFeature::Folder;
}

void KmlRunner::parseFile( const QString &fileName, DocumentRole role = UnknownDocument )
{
    QFile  file( fileName );
    if ( !file.exists() ) {
        qWarning( "File does not exist!" );
        return;
    }

    // Open file in right mode
    file.open( QIODevice::ReadOnly );

    KmlParser parser;

    if ( !parser.read( &file ) ) {
        emit parsingFinished( 0, parser.errorString() );
        return;
    }
    GeoDocument* document = parser.releaseDocument();
    Q_ASSERT( document );
    GeoDataDocument* doc = static_cast<GeoDataDocument*>( document );
    doc->setDocumentRole( role );

	GeoDataContainer *container =  doc;
	const QVector<GeoDataPlacemark*> placemarks = container->placemarkList();
	
//	createPlacemarks(container);

    file.close();
    emit parsingFinished( doc );
}

void KmlRunner::createPlacemarks( GeoDataContainer *container )
{
	qreal lon=0;
    qreal lat=0;
    qreal alt=0;

	GeoDataCoordinates pos;

	const char* chType=NULL;
	const char* chFolder = GeoDataTypes::GeoDataFolderType;
	const char* chDoc = GeoDataTypes::GeoDataDocumentType;

	QString qsFolder(chFolder);
	QString qsDoc(chDoc);

    QVector<GeoDataFeature*>::Iterator i = container->begin();
    QVector<GeoDataFeature*>::Iterator const end = container->end();
    for (; i != end; ++i ) 
	{
		chType = (*i)->nodeType();
		QString ad = (*i)->address();
		ad = (*i)->description();
		Marble::GeoDataExtendedData exd = (*i)->extendedData();
		const char* chT = exd.nodeType();
		ad = (*i)->name();
		ad = (*i)->styleUrl();
		QString qsType(chType);

        if ( qsType == qsFolder
             || qsType == qsDoc ) 
		{
            GeoDataContainer *child = static_cast<GeoDataContainer*>( *i );
            createPlacemarks( child );
        } else {
            GeoDataPlacemark* placemark = static_cast<GeoDataPlacemark*>( *i );

			if( dynamic_cast<GeoDataLineString*>( placemark->geometry() ) )
			{
				GeoDataLineString* gdl = dynamic_cast<GeoDataLineString*>( placemark->geometry() );
				QString qs = placemark->name();
				qs = placemark->styleUrl();
			}
			else if( dynamic_cast<GeoDataPolygon*>( placemark->geometry() ) ) 
			{
				GeoDataPolygon* gdl = dynamic_cast<GeoDataPolygon*>( placemark->geometry() );
				QString qs = placemark->name();
				qs = placemark->styleUrl();

			}
			else if( dynamic_cast<GeoDataIconStyle*>( placemark->geometry() ) ) 
			{
				pos = placemark->coordinate();
				QString qs = placemark->name();
				qs = placemark->styleUrl();
			
			}
			else if( dynamic_cast<GeoDataPoint*>( placemark->geometry() ) ) 
			{
				placemark->coordinate( lon, lat, alt );
				QString qs = placemark->name();
				qs = placemark->styleUrl();
				const GeoDataStyle* style = placemark->style();
				qs = style->iconStyle().iconPath();
			}

		}
	}
}

}

#include "moc_KmlRunner.cpp"
