/*******************************************************************************
* @file			MapViewDimLayer.cpp
* @copyright	201-2012 Rohde&Schwarz, Munich
* @version		$Header: MapViewDimLayer.h
*
* @author		Konrad Enzensberger	e.konrad@mpegcode.com
* @responsible	KOnrad Enzensberger
* 
* @language		C++ ANSI V3
*
* @history
*
*
********************************************************************************
*******************************************************************************/

/* INCLUDE FILES **************************************************************/

#include "MapViewDimLayer.h"

#include "GeoDataCoordinates.h" 
#include "GeoDataLineString.h"
#include "GeoSceneDocument.h"
#include "GeoSceneHead.h"
#include "GeoPainter.h"
#include "MarbleWidget.h"
#include "MarbleMap.h"
#include "TreeViewModel.h"

#include <QtCore/QMap>
#include <QtGui/QIcon>
#include <QtGui/QKeyEvent>
#include <QtGui/QMouseEvent>
#include <QtGui/QPixmap>
#include <QtGui/QImage>
#include <QtGui/QItemSelectionModel>

namespace Marble
{

class MapViewDimLayerPrivate
{
    template<class T>
    struct PaintRegion {
        T index;
        QRegion region;

        PaintRegion( const T &index_, const QRegion &region_ ) :
                index( index_ ), region( region_ )
        {
            // nothing to do
        }
    };

    typedef PaintRegion<QModelIndex> ModelRegion;
    typedef PaintRegion<int> RequestRegion;

public:
    MapViewDimLayer *q;

	qint64 m_indexID;

	bool m_viewportChanged;

    MarbleWidget *m_marbleWidget;

 //   QPixmap m_targetPixmap;
	QMap<long,QImage>	m_qmImage;

    QRect m_dirtyRect;

	QItemSelectionModel *m_selectionModel;
    TreeViewModel	*m_treeviewModel;
	TreeNode		*m_pNode;

    bool m_layerDirty;

    QSize m_pixmapSize;
	int		m_iDimmer;

    /** Constructor */
    explicit MapViewDimLayerPrivate( MapViewDimLayer *parent, MarbleWidget *widget, qint64 index );

    /** Paint waypoint polygon */
    inline void renderLayer( GeoPainter *painter );

	/** Insert via points or emit position signal, if appropriate */
    inline bool handleMouseButtonRelease( QMouseEvent *e );

    /** Select route instructions points, start dragging trip points */
    inline bool handleMouseButtonPress( QMouseEvent *e );

    /** Dragging trip points, route polygon hovering */
    inline bool handleMouseMove( QMouseEvent *e );

    /** Escape to stop selecting points */
    inline bool handleKeyEvent( QKeyEvent *e );

	inline bool calculateImageCoord(QImage& img,const GeoDataCoordinates& lt,const GeoDataCoordinates& rb,const int& width,const int& heigth,const int& iGrid,const int& iID);


};

MapViewDimLayerPrivate::MapViewDimLayerPrivate( MapViewDimLayer *parent, MarbleWidget *widget, qint64 index ) :
        q( parent ),  m_marbleWidget( widget ),
        m_treeviewModel( 0 ),m_pNode(0), m_selectionModel( 0 ),
        m_layerDirty( false ), m_pixmapSize( 22, 22 ),
		m_indexID(index), m_viewportChanged(true)
{
   
	m_iDimmer=0;

}


void MapViewDimLayerPrivate::renderLayer( GeoPainter *painter )
{
  //  GeoDataLineString waypoints;

	if(m_iDimmer==0)
		return;

	QRegion rg = m_marbleWidget->mapRegion();
	QRect qr = rg.boundingRect();

	bool darken=true;
	if(m_iDimmer<0)
	{
		darken=false;
	}

	int iVal = (int)((255.0/100)*(qreal)m_iDimmer);
	if(!darken)
		iVal = (int)((255.0/100)*(qreal)(m_iDimmer*-1) );
	//iVal = 255-iVal;
	QColor color;

	if(darken)
		color = QColor( 0, 0,0, iVal );
	else
		color = QColor(255,255,255, iVal );

	QPen layerPen(color);
    layerPen.setWidth( 1 );
	painter->setPen(layerPen);
	painter->setBrush(color);

	painter->drawRect(qr.left(),qr.top(),qr.width(),qr.height());

}

bool MapViewDimLayerPrivate::calculateImageCoord(QImage& qImg,const GeoDataCoordinates& lt,const GeoDataCoordinates& rb,const int& width,const int& height,const int& iGrid,const int& iID)
{
	bool ret=false;

	QImage qImage = m_qmImage.value((long)iID);
	if(qImage.isNull())return ret;

	if(qImage.width()>=width && qImage.height()>=height)
	{
		qImg = qImage.copy();
		return true;
	}

	QRgb rgb;

	qreal dMinX = lt.longitude(GeoDataCoordinates::Degree);
	qreal dMaxY = lt.latitude(GeoDataCoordinates::Degree);
	qreal dMaxX = rb.longitude(GeoDataCoordinates::Degree);
	qreal dMinY = rb.latitude(GeoDataCoordinates::Degree);

	qreal dX = (rb.longitude(GeoDataCoordinates::Degree)-lt.longitude(GeoDataCoordinates::Degree))/iGrid;
	qreal dY = (lt.latitude(GeoDataCoordinates::Degree)-rb.latitude(GeoDataCoordinates::Degree))/iGrid;
	qreal dLon =0;
	qreal dLat =0;

	qreal x1=0,y1=0,x2=0,y2=0;
	int iCount = iGrid*iGrid;
	int iX=0;
	int iY=0;
	int inX=0;
	int inY=0;

	m_marbleWidget->screenCoordinates(dMinX,dMaxY,x1,y1);
	m_marbleWidget->screenCoordinates(dMaxX,dMinY,x2,y2);

	inX = (int)(x1+0.5);
	inY = (int)(y1+0.5);

	int ixpos1=0;
	int ixpos2=0;
	int iypos1=0;
	int iypos2=0;
	for(int i=0;i<iCount;i++)
	{
		dLon = dMinX + (iX*dX); 
		dLat = dMaxY - (iY*dY); 
		m_marbleWidget->screenCoordinates(dLon,dLat,x1,y1);
		m_marbleWidget->screenCoordinates(dLon+dX,dLat-dY,x2,y2);

		ixpos1 = (int)(x1+0.5);
		ixpos2 = (int)(x2+0.5);
		iypos1 = (int)(y1+0.5);
		iypos2 = (int)(y2+0.5);

		rgb = qImage.pixel(iX,iY);
		for(int y=iypos1;y<=iypos2;y++)
		{
			for(int x=ixpos1;x<=ixpos2;x++)
			{
				qImg.setPixel(x-inX,y-inY,rgb);
			//	inX++;
			/*	if(x==width)
				{
					x=0;
					y++;
				}*/
				//if(inY==height)
				//	inY=0;

			}
		}

		iX++;
		if(iX==iGrid)
		{
			iX=0;
			iY++;
		}
	}

//	m_qmImage.remove((long)iID);
//	m_qmImage.insert((long)iID,qImg);
	ret=true;

	return ret;

}

/** Insert via points or emit position signal, if appropriate */
bool MapViewDimLayerPrivate::handleMouseButtonRelease( QMouseEvent *e )
{
	 return false;
}

/** Select route instructions points, start dragging trip points */
bool MapViewDimLayerPrivate::handleMouseButtonPress( QMouseEvent *e )
{
	 return false;
}

    /** Dragging trip points, route polygon hovering */
bool MapViewDimLayerPrivate::handleMouseMove( QMouseEvent *e )
{
	 return false;
}

    /** Escape to stop selecting points */
bool MapViewDimLayerPrivate::handleKeyEvent( QKeyEvent *e )
{
	 return false;
}

MapViewDimLayer::MapViewDimLayer( MarbleWidget *widget, QWidget *parent, qint64 index ) :
        QObject( parent ), d( new MapViewDimLayerPrivate( this, widget,index ) )
{
    widget->installEventFilter( this );

	connect( widget, SIGNAL( visibleLatLonAltBoxChanged( GeoDataLatLonAltBox ) ),
            this, SLOT( setViewportChanged() ) );
}

MapViewDimLayer::~MapViewDimLayer()
{
	d->m_qmImage.clear();
    delete d;
}

QStringList MapViewDimLayer::renderPosition() const
{
    //return QStringList() << "HOVERS_ABOVE_SURFACE";
	//return QStringList() << "ALWAYS_ON_TOP";
	return QStringList() << "SURFACE";
}

bool MapViewDimLayer::render( GeoPainter *painter, ViewportParams *viewport,
                           const QString& renderPos, GeoSceneLayer *layer )
{
    if ( d->m_marbleWidget->mapTheme()->head()->target() != "earth" ) {
        return false;
    }

    Q_UNUSED( viewport )
    Q_UNUSED( renderPos )
    Q_UNUSED( layer )

    painter->save();
    painter->autoMapQuality();

    d->renderLayer( painter );

    painter->restore();
	// d->m_viewportChanged = false;
    return true;
}

bool MapViewDimLayer::eventFilter( QObject *obj, QEvent *event )
{
    Q_UNUSED( obj )

    if ( event->type() == QEvent::KeyPress ) {
        QKeyEvent *e = static_cast<QKeyEvent*>( event );
        return d->handleKeyEvent( e );
    }

    if ( event->type() == QEvent::MouseButtonPress ) {
        QMouseEvent *e = static_cast<QMouseEvent*>( event );
        return d->handleMouseButtonPress( e );
    }

    if ( event->type() == QEvent::MouseButtonRelease ) {
        QMouseEvent *e = static_cast<QMouseEvent*>( event );
        return d->handleMouseButtonRelease( e );
    }

    if ( event->type() == QEvent::MouseMove ) {
        QMouseEvent *e = static_cast<QMouseEvent*>( event );
        return d->handleMouseMove( e );
    }

    return false;
}

void MapViewDimLayer::setModel ( TreeViewModel *model )
{
    d->m_treeviewModel = model;
	setViewportChanged();
}

void MapViewDimLayer::synchronizeWith( QItemSelectionModel *selection )
{
    d->m_selectionModel = selection;
//    d->m_proxyModel = model;
}


void MapViewDimLayer::setLayerDirty( bool dirty )
{
    d->m_layerDirty = dirty;

    /** @todo: The full repaint can be avoided. The route however has changed
      * and the exact bounding box needs to be recalculated before doing
      * a partly repaint, otherwise we might end up repainting only parts of the route
      */
    // d->m_marbleWidget->repaint( d->m_routeRegion );
    d->m_marbleWidget->update();
}


void MapViewDimLayer::setViewportChanged()
{
    d->m_viewportChanged = true;
}

void MapViewDimLayer::updateImageList (long id, QImage *img )
{
	if(img!=NULL)
	{
		QImage qImg = img->copy();
		d->m_qmImage.insert(id,qImg);
	}
	else
		d->m_qmImage.remove(id);
}

void MapViewDimLayer::setDimmerValue(int dValue)
{
	if ( d!=NULL)
	{
		d->m_iDimmer=dValue;
		 d->m_marbleWidget->update();
	}
}

// Overriding LayerInterface to paint on top of the route
qreal MapViewDimLayer::zValue() const
{
	return 1.0;
}

} // namespace Marble

#include "moc_MapViewDimLayer.cpp"
