/*******************************************************************************
* @file			MapViewMapLayer.cpp
* @copyright	201-2012 Rohde&Schwarz, Munich
* @version		$Header: MapViewMapLayer.h
*
* @author		Konrad Enzensberger		e.konrad@mpegcode.com
* @responsible	KOnrad Enzensberger
* 
* @language		C++ ANSI V3
*
* @history
*
*
********************************************************************************
*******************************************************************************/

/* INCLUDE FILES **************************************************************/

#include "MapViewMapLayer.h"

#include "GeoDataTypes.h"
#include "GeoDataDocument.h"
#include "GeoDataGroundOverlay.h"
#include "GeoDataCoordinates.h" 
#include "GeoDataLineString.h"
#include "GeoSceneDocument.h"
#include "GeoSceneHead.h"
#include "GeoPainter.h"
#include "GeoDataTreeModel.h"
#include "MarbleWidget.h"
#include "MarbleMap.h"
#include "MarbleModel.h"
#include "TreeViewModel.h"
#include "planet.h"
#include "ViewportParams.h"

#include <QtCore/QMap>
#include <QtGui/QIcon>
#include <QtGui/QKeyEvent>
#include <QtGui/QMouseEvent>
#include <QtGui/QPixmap>
#include <QtGui/QImage>
#include <QtGui/QItemSelectionModel>
#include <QFileDialog>

namespace Marble
{

class MapViewMapLayerPrivate
{
    template<class T>
    struct PaintRegion {
        T index;
        QRegion region;

        PaintRegion( const T &index_, const QRegion &region_ ) :
                index( index_ ), region( region_ )
        {
            // nothing to do
        }
    };

    typedef PaintRegion<QModelIndex> ModelRegion;
    typedef PaintRegion<int> RequestRegion;

public:
    MapViewMapLayer *q;
	QList<RequestRegion>	m_regions;		//GroundOverlay region

	qint64 m_indexID;

	qreal m_dLongitude;
	qreal m_dLatitude;

    MarbleWidget*		m_marbleWidget;
	GeoDataDocument*	m_pMap;
	int					m_iFeatureIndex;
	bool				m_isFeature;

	bool				m_bGroundOverlay;
	bool				m_viewportChanged;
	bool				m_bInputEnabled;	//if true, do not take care obout mouse move over objects
	bool				m_bObject;

    QRect				m_dirtyRect;
	QPoint				m_mousePos;

	QItemSelectionModel *m_selectionModel;
    TreeViewModel		*m_treeviewModel;
	TreeNode			*m_pNode;

	QString				m_layerName;
    bool				m_layerDirty;

    QSize				m_pixmapSize;
	GeoDataLatLonBox	m_goBox;
	int					m_imageWidth;
	int					m_imageHeight;

    /** Constructor */
    explicit MapViewMapLayerPrivate( MapViewMapLayer *parent, MarbleWidget *widget, qint64 index );

    /** Paint waypoint polygon */
    inline void renderLayer( GeoPainter *painter );

	/** Insert via points or emit position signal, if appropriate */
    inline bool handleMouseButtonRelease( QMouseEvent *e );

    /** Select route instructions points, start dragging trip points */
    inline bool handleMouseButtonPress( QMouseEvent *e );

    /** Dragging trip points, route polygon hovering */
    inline bool handleMouseMove( QMouseEvent *e );

    /** Escape to stop selecting points */
    inline bool handleKeyEvent( QKeyEvent *e );

	 /** True if the given point (screen coordinates) is among the route instruction points */
    inline bool isInfoPoint( const QPoint &point );

	/**
	  * convert pixel pos to geo coordinates - check if pixel position within GroundOverlay
	  */
	inline bool pixelToCoord(int xpos,int ypos, double& lon,double& lat);

	/**
	  * convert geo coordinates to pixel pos - check if pixel position within GroundOverlay
	  */
	inline bool coordToPixel(int& xpos,int& ypos, double lon,double lat);

	
	/**
	  * add document to model
	  */
	void addDocument(GeoDataDocument* pDoc,bool bGOV=false);

	/**
	  * add document by zval
	  */
	void addDocument(qreal zval);
	/**
	  * remove document from model
	  */
	void removeDocument();

};

MapViewMapLayerPrivate::MapViewMapLayerPrivate( MapViewMapLayer *parent, MarbleWidget *widget, qint64 index ) :
        q( parent ),  m_marbleWidget( widget ),
        m_treeviewModel( 0 ),m_pNode(0), m_selectionModel( 0 ),
        m_layerDirty( false ), m_pixmapSize( 22, 22 ),m_iFeatureIndex(-1),
		m_indexID(index), m_viewportChanged(true),m_pMap(NULL),m_bGroundOverlay(false),m_bObject(false),m_bInputEnabled(false),m_isFeature(false)
{
		m_dLongitude= -399.0;
		m_dLatitude= -390.0;
		m_imageWidth=0;
		m_imageHeight=0;
}

void MapViewMapLayerPrivate::addDocument(GeoDataDocument* pDoc,bool bGOV)
{
	if(m_pMap==NULL&&pDoc==NULL)return;
	m_isFeature=false;

	if(m_pMap==NULL)
	{
		const char* chType=NULL;
		m_pMap = pDoc;
		m_pMap->setVisible(true);
		m_pMap->setZoomLevel((int)q->m_zValue);

		m_isFeature=bGOV;
		
		if(0)//bGOV)
		{
			GeoDataFeature* pFeature = static_cast<GeoDataFeature*>(pDoc);
			GeoDataGroundOverlay* pOvl = static_cast<GeoDataGroundOverlay*>(pFeature);
			if(pOvl!=NULL)
			{
				m_isFeature=true;
				m_goBox = pOvl->latLonBox();
				QString str = pOvl->absoluteIconFile();
				QImage img;
				if(img.load(str))
				{
					m_imageWidth=img.width();
					m_imageHeight=img.height();
				}
				else
				{
					QFile imgFile( str );
					if ( !imgFile.exists() ) 
					{
						QString filename = q->getFileName();
						QFileInfo filePath(filename);
						QString path = filePath.absolutePath();
						QFileInfo fileInf(str);
						QString iconname = fileInf.fileName();
						path = path + '/' + iconname;
						QImage img;
						if(img.load(path))
						{
							pOvl->setIconFile(path);
							m_imageWidth=img.width();
							m_imageHeight=img.height();

							//m_marbleWidget->model()->treeModel()->removeDocument(m_pMap);
						}
					}
				}

				m_pMap = new GeoDataDocument();
				m_pMap->setVisible(true);
				m_pMap->setZoomLevel((int)q->m_zValue);
				m_pMap->append(pFeature);
				m_iFeatureIndex = m_marbleWidget->model()->treeModel()->addDocument(m_pMap);
			}//pOvl
			m_bGroundOverlay=true;
			return;
		}//bGov

		m_iFeatureIndex = m_marbleWidget->model()->treeModel()->addDocument(pDoc);

	//	if ( const GeoDataOverlay* overlay = dynamic_cast<const GeoDataOverlay*>( pDoc ) )
		GeoDataContainer *container =  dynamic_cast<GeoDataContainer*>(pDoc);
	//	if(object==NULL)return;
		QVector<GeoDataFeature*>::Iterator i = container->begin();
		QVector<GeoDataFeature*>::Iterator const end = container->end();
		for (; i != end; ++i ) 
		{
			chType = (*i)->nodeType();
			if( chType == GeoDataTypes::GeoDataFolderType)
			{
				GeoDataContainer *child = static_cast<GeoDataContainer*>( *i );
				QVector<GeoDataFeature*>::Iterator ic = child->begin();
				QVector<GeoDataFeature*>::Iterator const endc = child->end();
				for (; ic != endc; ++ic ) 
				{
					chType = (*ic)->nodeType();
				
					if( chType == GeoDataTypes::GeoDataGroundOverlayType)
					{
						GeoDataGroundOverlay *overlay =  dynamic_cast<GeoDataGroundOverlay*>(*ic);
						if(overlay!=NULL)
						{
							m_goBox = overlay->latLonBox();
							QString str = overlay->absoluteIconFile();
							QImage img;
							if(img.load(str))
							{
								m_imageWidth=img.width();
								m_imageHeight=img.height();
							}
							else
							{
								QFile imgFile( str );
								if ( !imgFile.exists() ) 
								{
									QString filename = pDoc->fileName();
									QFileInfo filePath(filename);
									QString path = filePath.absolutePath();
									QFileInfo fileInf(str);
									QString iconname = fileInf.fileName();
									path = path + '/' + iconname;
									QImage img;
									if(img.load(path))
									{
										overlay->setIconFile(path);
										m_imageWidth=img.width();
										m_imageHeight=img.height();

										if(bGOV)
											pDoc->setFileName(path);

										m_marbleWidget->model()->treeModel()->removeDocument(m_pMap);
										m_iFeatureIndex = m_marbleWidget->model()->treeModel()->addDocument(pDoc);
									}
								}
							}
						}
						m_bGroundOverlay=true;
					}
					else if( chType == GeoDataTypes::GeoDataPlacemarkType)
					{
						GeoDataPlacemark *placemark =  dynamic_cast<GeoDataPlacemark*>(*ic);
						if(placemark!=NULL)
						{
							qreal alt=0;
							placemark->coordinate(m_dLongitude,m_dLatitude,alt);
						}

					}
					else return;
				}

				return;
			}//GeoDataFolder
			else if( chType == GeoDataTypes::GeoDataGroundOverlayType)//import tab: root element is ground overly
			{
				GeoDataGroundOverlay *overlay =  dynamic_cast<GeoDataGroundOverlay*>(*i);
				if(overlay!=NULL)
				{
					m_goBox = overlay->latLonBox();
					QString str = overlay->absoluteIconFile();
					QImage img;
					if(img.load(str))
					{
						m_imageWidth=img.width();
						m_imageHeight=img.height();
					}
					else
					{
						QFile imgFile( str );
						if ( !imgFile.exists() ) 
						{
							QString filename = pDoc->fileName();
							QFileInfo filePath(filename);
							QString path = filePath.absolutePath();
							QFileInfo fileInf(str);
							QString iconname = fileInf.fileName();
							path = path + '/' + iconname;
							QImage img;
							if(img.load(path))
							{
								overlay->setIconFile(path);
								m_imageWidth=img.width();
								m_imageHeight=img.height();

								if(bGOV)
									pDoc->setFileName(path);

								m_marbleWidget->model()->treeModel()->removeDocument(m_pMap);
								m_iFeatureIndex = m_marbleWidget->model()->treeModel()->addDocument(pDoc);
							}
						}

					}
				}
				m_bGroundOverlay=true;
			}
			else if( chType == GeoDataTypes::GeoDataPlacemarkType)
			{
				GeoDataPlacemark *placemark =  dynamic_cast<GeoDataPlacemark*>(*i);
				if(placemark!=NULL)
				{
					qreal alt=0;
					placemark->coordinate(m_dLongitude,m_dLatitude,alt);
				}
			}
		}


	//	const GeoDataOverlay* overlay = dynamic_cast<const GeoDataOverlay*>( object );
	//	if(overlay==NULL)return;
	//	if(object->nodeType() == GeoDataTypes::GeoDataGroundOverlayType )
	//		m_bGroundOverlay=true;
	}
	else
	{
		m_marbleWidget->model()->treeModel()->removeDocument(m_pMap);
		m_iFeatureIndex=-1;
	}

}

void MapViewMapLayerPrivate::addDocument(qreal zval)
{
	if(m_pMap==NULL)return;
	//if(!m_bGroundOverlay)return;

	QModelIndex idxParent = m_treeviewModel->getIndexFromIndexId(m_indexID);
	if(!idxParent.isValid())return;

	QVariant qVar = m_treeviewModel->data(idxParent,Qt::CheckStateRole);
	bool bVisible = qVar.toBool();

	m_pMap->setVisible(bVisible);
	m_pMap->setZoomLevel((int)q->m_zValue);
	m_pMap->setPopularity((qint64)q->m_zValue);
	m_iFeatureIndex=m_marbleWidget->model()->treeModel()->addDocument(m_pMap);
}
	
void MapViewMapLayerPrivate::removeDocument()
{
	if(m_pMap==NULL)return;
	//if(!m_bGroundOverlay)return;

	m_marbleWidget->model()->treeModel()->removeDocument(m_pMap);
	m_iFeatureIndex=-1;
}

void MapViewMapLayerPrivate::renderLayer( GeoPainter *painter )
{
//	QRegion rg = m_marbleWidget->mapRegion();
//	QRect qr = rg.boundingRect();
	if(m_pMap==NULL)return;

	QModelIndex idxParent = m_treeviewModel->getIndexFromIndexId(m_indexID);
	if(!idxParent.isValid())return;

	QVariant qVar = m_treeviewModel->data(idxParent,Qt::CheckStateRole);
	bool bVisible = qVar.toBool();

	if(/*!m_bGroundOverlay &&*/ bVisible && !m_pMap->isVisible())
	{
	//	m_marbleWidget->model()->treeModel()->removeDocument(m_pMap);
	//	GeoDataFeature* pFeature = m_marbleWidget->model()->treeModel()->f
		m_pMap->setVisible(bVisible);
		m_pMap->setZoomLevel((int)q->m_zValue);
		m_marbleWidget->model()->treeModel()->updateFeature(m_pMap);
		//m_marbleWidget->model()->treeModel()->addDocument(m_pMap);
		
	}
/*	else if(m_bGroundOverlay && bVisible && !m_pMap->isVisible())
	{
		m_marbleWidget->model()->treeModel()->removeDocument(m_pMap);
		m_pMap->setVisible(bVisible);
		m_pMap->setZoomLevel((int)q->m_zValue);
		m_marbleWidget->model()->treeModel()->addDocument(m_pMap);
	}*/
	else if(!bVisible && m_pMap->isVisible())
	{
		m_pMap->setVisible(bVisible);
	//	m_marbleWidget->model()->treeModel()->removeDocument(m_pMap);
		m_marbleWidget->model()->treeModel()->updateFeature(m_pMap);
	}

	if(m_bGroundOverlay && m_viewportChanged)
	{
		m_regions.clear();
		qreal north,south,east,west;
		m_goBox.boundaries(north,south,east,west);
		GeoDataLinearRing gdl;
		GeoDataCoordinates pos(west,north);
		gdl << pos;
		pos.set(east,north);
		gdl << pos;
		pos.set(east,south);
		gdl << pos;
		pos.set(west,south);
		gdl << pos;
		QRegion region = painter->regionFromPolygon(gdl,Qt::WindingFill);
		if(!region.isEmpty())
			m_regions.push_front( RequestRegion( 1, region ) );
	}

	m_viewportChanged=false;
}

/** Insert via points or emit position signal, if appropriate */
bool MapViewMapLayerPrivate::handleMouseButtonRelease( QMouseEvent *e )
{
	if(m_bInputEnabled)
		return false;

	if(!m_bObject)
		return false;

	QPoint mousePos = m_mousePos;
	m_mousePos.setX(-1);
	m_mousePos.setY(-1);

	//click event missing
	if(mousePos.x()==-1 && mousePos.y()==-1)
		return false;

	QModelIndex idxParent = m_treeviewModel->getIndexFromIndexId(m_indexID);
	if(!idxParent.isValid())return false;

	QVariant qVar = m_treeviewModel->data(idxParent,Qt::CheckStateRole);
	bool bVisible = qVar.toBool();

	if(bVisible)
	{
		QPoint pt = e->pos();
		//mouse moved ?
		if( abs(mousePos.x()-pt.x())>5 || abs(mousePos.y()-pt.y())>5)
			return false;

		qreal lon( 0.0 ), lat( 0.0 );
		m_marbleWidget->geoCoordinates( pt.x(),pt.y(),lon, lat, GeoDataCoordinates::Degree ); 

		qreal north=0,south=0,east=0,west=0;
		m_goBox.boundaries(north,south,east,west,GeoDataCoordinates::Degree);

		//screen coordinates from GroundOverlay bounding box
		qreal left=0,top=0,right=0,bottom=0;
		m_marbleWidget->screenCoordinates(west,north,left,top);
		m_marbleWidget->screenCoordinates(east,south,right,bottom);

		//length in meter from left to right
		GeoDataCoordinates pos1(west,north,0,GeoDataCoordinates::Degree);
		GeoDataCoordinates pos2(east,north,0,GeoDataCoordinates::Degree);
		GeoDataLineString lr1;
		lr1 << pos1 << pos2;
		qreal lenX = lr1.length(m_marbleWidget->model()->planet()->radius());
		lenX = lenX-((lenX/100.0)*2);

		//meter per pixel in x direction
		qreal meterPerPixX = lenX/(qreal)m_imageWidth;

		//length in meter from top to bottom
		GeoDataCoordinates pos3(west,north,0,GeoDataCoordinates::Degree);
		GeoDataCoordinates pos4(west,south,0,GeoDataCoordinates::Degree);
		GeoDataLineString lr2;
		lr2 << pos3 << pos4;
		qreal lenY = lr2.length(m_marbleWidget->model()->planet()->radius());
		lenY = lenY-((lenY/100.0)*2);

		//meter per pixel in y direction
		qreal meterPerPixY = lenY/(qreal)m_imageHeight;

		lr1.clear();
		//length in meter from left to click x position
		pos1.set(west,lat,0,GeoDataCoordinates::Degree);
		pos2.set(lon,lat,0,GeoDataCoordinates::Degree);
		lr1 << pos1 << pos2;
		lenX = lr1.length(m_marbleWidget->model()->planet()->radius());
		lenX = lenX-((lenX/100.0)*2);

		//image x position
		lenX = lenX/meterPerPixX;

		lr2.clear();
		//length in meter from bottom to click y position
		pos3.set(lon,south,0,GeoDataCoordinates::Degree);
		pos4.set(lon,lat,0,GeoDataCoordinates::Degree);
		lr2 << pos3 << pos4;
		lenY = lr2.length(m_marbleWidget->model()->planet()->radius());
		lenY = lenY-((lenY/100.0)*2);

		//image y position
		lenY = lenY/meterPerPixY;
		//top-down coordinates
		lenY = m_imageHeight-lenY;
		if(lenY<0)lenY=0;

		QPoint imgPt;
		imgPt.setX((int)lenX);
		imgPt.setY((int)lenY);

		emit q->signalNotifyImageMouseAction(pt,imgPt,101,q->m_InterfaceID);//left button
		return true; //to be able to pan the map
	}

	return false;
}

/** Select route instructions points, start dragging trip points */
bool MapViewMapLayerPrivate::handleMouseButtonPress( QMouseEvent *e )
{
	m_mousePos.setX(-1);
	m_mousePos.setY(-1);

	if(m_bInputEnabled)
		return false;
	if(e->buttons() & Qt::RightButton)
		return false;

	if(!m_bObject)return false;

	m_mousePos = e->pos();

	return false;
}

    /** Dragging trip points, route polygon hovering */
bool MapViewMapLayerPrivate::handleMouseMove( QMouseEvent *e )
{
	if(m_bInputEnabled)
		return false;

	if(e->buttons() & Qt::LeftButton)
		return false;
	if(e->buttons() & Qt::RightButton)
		return false;

	m_bObject=false;
	if(!m_bGroundOverlay)return false;

	qreal lon( 0.0 ), lat( 0.0 );
    if ( m_marbleWidget->geoCoordinates( e->pos().x(), e->pos().y(),
                                         lon, lat, GeoDataCoordinates::Radian ) ) 
	{
		 if ( isInfoPoint( e->pos() ) )
		 {
			m_marbleWidget->setCursor( Qt:: PointingHandCursor );
			m_bObject=true;
			QString position = GeoDataCoordinates( lon, lat ).toString();
            emit q->mouseMoveGeoPosition( position );
			return true;
		 }
	}
	 
	return false;
}

    /** Escape to stop selecting points */
bool MapViewMapLayerPrivate::handleKeyEvent( QKeyEvent *e )
{
	if(m_bInputEnabled)
		return false;

	 return false;
}

bool MapViewMapLayerPrivate::isInfoPoint( const QPoint &point )
{
    foreach( const RequestRegion &region, m_regions ) 
	{
        if ( region.region.contains( point ) ) {
			//m_infoID = region.index;
            return true;
        }
    }

	return false;
}

bool MapViewMapLayerPrivate::pixelToCoord(int xpos,int ypos, double& lon,double& lat)
{
	bool ret = false;
	
	//GroundOverlay bounding box
	qreal north=0,south=0,east=0,west=0;
	m_goBox.boundaries(north,south,east,west,GeoDataCoordinates::Degree);

	qreal left=0,top=0,right=0,bottom=0;
	m_marbleWidget->screenCoordinates(west,north,left,top);
	m_marbleWidget->screenCoordinates(east,south,right,bottom);

	qreal deltax = (right-left)/(qreal)m_imageWidth;
	qreal deltay = (bottom-top)/(qreal)m_imageHeight;

	int screenx = left+(xpos * deltax);
	int screeny = bottom-((m_imageHeight-ypos) * deltay);

	QPoint pt(screenx,screeny);
	ret = isInfoPoint(pt);
	if(ret)//calculated point within image?
		m_marbleWidget->geoCoordinates(screenx,screeny,lon,lat);
	else
		m_marbleWidget->geoCoordinates(xpos,ypos,lon,lat);

	return true;

	if(xpos>left || xpos>right)return false;
	if(xpos>m_imageWidth)return false;
	if(ypos>m_imageHeight)return false;

	qreal screenLeft=0;
	qreal screenTop=0;
	qreal screenRight=0;
	qreal screenBottom=0;

	GeoDataLineString lr1;
	GeoDataLineString lr2;
	//meter per screen pixel
	qreal distance = m_marbleWidget->distance()*1000;//meter
	int height = m_marbleWidget->viewport()->height();
	int width = m_marbleWidget->viewport()->width(); 
	distance = distance/(qreal)width;					//meter per pixel

	m_marbleWidget->geoCoordinates(0,0,screenLeft,screenTop);
	m_marbleWidget->geoCoordinates(width,height,screenRight,screenBottom);
	
	GeoDataCoordinates scr1(screenLeft,screenTop-((screenTop-screenBottom)/2),0,GeoDataCoordinates::Degree);
	GeoDataCoordinates scr2(screenRight,screenTop-((screenTop-screenBottom)/2),0,GeoDataCoordinates::Degree);
	lr1 << scr1 << scr2;
	qreal scrX = lr1.length(m_marbleWidget->model()->planet()->radius());
	qreal dist1 = scrX/width;

	GeoDataCoordinates scr3(screenLeft+((screenRight-screenLeft)/2),screenTop,0,GeoDataCoordinates::Degree);
	GeoDataCoordinates scr4(screenLeft+((screenRight-screenLeft)/2),screenBottom,0,GeoDataCoordinates::Degree);
	lr2 << scr3 << scr4;
	qreal scrY = lr2.length(m_marbleWidget->model()->planet()->radius());
	qreal dist2 = scrY/height;

	lr1.clear();

	GeoDataCoordinates pos1(west,north/2,0,GeoDataCoordinates::Degree);
	GeoDataCoordinates pos2(east,north/2,0,GeoDataCoordinates::Degree);
	lr1 << pos1 << pos2;
	qreal lenX = lr1.length(m_marbleWidget->model()->planet()->radius());
	lenX = lenX-((lenX/100.0)*2);

	//image meter per pixel in x direction
	qreal meterPerPixX = lenX/(qreal)m_imageWidth;

	//calc meter from image 0 to x pos
	lenX = xpos*meterPerPixX;

	//calc screen x pos 
	width = left + (lenX/dist1);

	lr2.clear();
	GeoDataCoordinates pos3(west,north,0,GeoDataCoordinates::Degree);
	GeoDataCoordinates pos4(west,south,0,GeoDataCoordinates::Degree);
	lr2 << pos3 << pos4;
	qreal lenY = lr2.length(m_marbleWidget->model()->planet()->radius());
	//lenY = lenY-((lenY/100.0)*2);

	//image meter per pixel in y direction
	qreal meterPerPixY = lenY/(qreal)m_imageHeight;

	//calc meter from image 0 to y pos
	lenY = ypos*meterPerPixY;

	//calc screen y pos 
	height = bottom - (lenY/dist2);

/*	QPoint pt(width,height);
	ret = isInfoPoint(pt);
	if(ret)
	{
	//	m_marbleWidget->geoCoordinates(width,height,screenLeft,screenTop);
	//	if(!m_goBox.contains(GeoDataCoordinates(screenLeft,screenTop,GeoDataCoordinates::Degree)) )
	//		return false;

		m_marbleWidget->geoCoordinates(width,height,lon,lat, GeoDataCoordinates::Degree);
	
	}
	*/
	return ret;

}

bool MapViewMapLayerPrivate::coordToPixel(int& xpos,int& ypos, double lon,double lat)
{	
	bool ret = false;
	qreal xPos=0,yPos=0;
	m_marbleWidget->screenCoordinates(lon,lat,xPos,yPos);

	QPoint pt((int)xPos,(int)yPos);
	ret=isInfoPoint(pt);
	if(!ret)return ret;

	qreal north=0,south=0,east=0,west=0;
	m_goBox.boundaries(north,south,east,west,GeoDataCoordinates::Degree);

	//screen coordinates from GroundOverlay bounding box
	qreal left=0,top=0,right=0,bottom=0;
	m_marbleWidget->screenCoordinates(west,north,left,top);
	m_marbleWidget->screenCoordinates(east,south,right,bottom);
/*
	qreal deltax = (right-left)/(qreal)m_imageWidth;
	qreal deltay = (bottom-top)/(qreal)m_imageHeight;

	xpos= (xPos-left)/deltax;
	ypos= (bottom-yPos)/deltay;

	return true;*/

	//length in meter from left to right
	GeoDataCoordinates pos1(west,north,0,GeoDataCoordinates::Degree);
	GeoDataCoordinates pos2(east,north,0,GeoDataCoordinates::Degree);
	GeoDataLineString lr1;
	lr1 << pos1 << pos2;
	qreal lenX = lr1.length(m_marbleWidget->model()->planet()->radius());
	lenX = lenX-((lenX/100.0)*2);

	//meter per pixel in x direction
	qreal meterPerPixX = lenX/(qreal)m_imageWidth;

	//length in meter from top to bottom
	GeoDataCoordinates pos3(west,north,0,GeoDataCoordinates::Degree);
	GeoDataCoordinates pos4(west,south,0,GeoDataCoordinates::Degree);
	GeoDataLineString lr2;
	lr2 << pos3 << pos4;
	qreal lenY = lr2.length(m_marbleWidget->model()->planet()->radius());
	lenY = lenY-((lenY/100.0)*2);

	//meter per pixel in y direction
	qreal meterPerPixY = lenY/(qreal)m_imageHeight;

	lr1.clear();
	//length in meter from left to click x position
	pos1.set(west,lat,0,GeoDataCoordinates::Degree);
	pos2.set(lon,lat,0,GeoDataCoordinates::Degree);
	lr1 << pos1 << pos2;
	lenX = lr1.length(m_marbleWidget->model()->planet()->radius());
	lenX = lenX-((lenX/100.0)*2);

	//image x position
	lenX = lenX/meterPerPixX;

	lr2.clear();
	//length in meter from bottom to click y position
	pos3.set(lon,south,0,GeoDataCoordinates::Degree);
	pos4.set(lon,lat,0,GeoDataCoordinates::Degree);
	lr2 << pos3 << pos4;
	lenY = lr2.length(m_marbleWidget->model()->planet()->radius());
	lenY = lenY-((lenY/100.0)*2);

	//image y position
	lenY = lenY/meterPerPixY;
	//top-down coordinates
	lenY = m_imageHeight-lenY;
	if(lenY<0)lenY=0;

	xpos=(int)lenX;
	ypos=(int)lenY;

	return ret;
}

/****************************************************************************************/

MapViewMapLayer::MapViewMapLayer( MarbleWidget *widget, QWidget *parent, qint64 index ) :
        QObject( parent ), d( new MapViewMapLayerPrivate( this, widget,index ) )
{
    widget->installEventFilter( this );

	connect( widget, SIGNAL( visibleLatLonAltBoxChanged( GeoDataLatLonAltBox ) ),
            this, SLOT( setViewportChanged() ) );

	GeoDataTreeModel* pTreeModel=d->m_marbleWidget->model()->treeModel();
	if(pTreeModel!=NULL)
	{
		connect( pTreeModel, SIGNAL( removed( GeoDataObject*) ),
            this, SLOT( slotRemoved( GeoDataObject*) ) );

		connect( pTreeModel, SIGNAL( added( GeoDataObject*) ),
            this, SLOT( slotAdded( GeoDataObject*) ) );
	
	}

	connect( this, SIGNAL( mouseMoveGeoPosition( QString ) ),
             widget, SIGNAL( mouseMoveGeoPosition( QString ) ) );

	m_pObject=NULL;
	m_InterfaceID=0;
	m_zValue=1.0;
}

MapViewMapLayer::~MapViewMapLayer()
{
    delete d;
}

QStringList MapViewMapLayer::renderPosition() const
{
    return QStringList() << "HOVERS_ABOVE_SURFACE";
	//return QStringList() << "ALWAYS_ON_TOP";
	//return QStringList() << "SURFACE";
	//return QStringList() << "USER_TOOLS";
}

bool MapViewMapLayer::render( GeoPainter *painter, ViewportParams *viewport,
                           const QString& renderPos, GeoSceneLayer *layer )
{
    if ( d->m_marbleWidget->mapTheme()->head()->target() != "earth" ) {
        return false;
    }

	if ( renderPos != "USER_TOOLS" ) {
        return true;
    }

    Q_UNUSED( viewport )
    Q_UNUSED( renderPos )
    Q_UNUSED( layer )

    painter->save();
    painter->autoMapQuality();

    d->renderLayer( painter );

    painter->restore();
	// d->m_viewportChanged = false;
    return true;
}

bool MapViewMapLayer::eventFilter( QObject *obj, QEvent *event )
{
    Q_UNUSED( obj )

	QModelIndex idxLayer = d->m_treeviewModel->getIndexFromIndexId(d->m_indexID);
	if(!idxLayer.isValid())
		return false;
	QVariant qVar = d->m_treeviewModel->data(idxLayer,Qt::CheckStateRole);
	if(!qVar.toBool())
	{
		return false;
	}

    if ( event->type() == QEvent::KeyPress ) {
        QKeyEvent *e = static_cast<QKeyEvent*>( event );
        return d->handleKeyEvent( e );
    }

    if ( event->type() == QEvent::MouseButtonPress ) {
        QMouseEvent *e = static_cast<QMouseEvent*>( event );
        return d->handleMouseButtonPress( e );
    }

    if ( event->type() == QEvent::MouseButtonRelease ) {
        QMouseEvent *e = static_cast<QMouseEvent*>( event );
        return d->handleMouseButtonRelease( e );
    }

    if ( event->type() == QEvent::MouseMove ) {
        QMouseEvent *e = static_cast<QMouseEvent*>( event );
        return d->handleMouseMove( e );
    }

    return false;
}

void MapViewMapLayer::slotRemoved( GeoDataObject *object )
{
	m_pObject=NULL;
}
    
void MapViewMapLayer::slotAdded( GeoDataObject *object )
{
	m_pObject=object;
}

void MapViewMapLayer::setModel ( TreeViewModel *model )
{
    d->m_treeviewModel = model;

	TreeNode* pNode = d->m_treeviewModel->getNodeFromIndexId(d->m_indexID);
	if(pNode!=NULL)
		d->m_layerName = pNode->m_qsName;

	setViewportChanged();
}

void MapViewMapLayer::synchronizeWith( QItemSelectionModel *selection )
{
    d->m_selectionModel = selection;
//    d->m_proxyModel = model;
}


void MapViewMapLayer::setLayerDirty( bool dirty )
{
    d->m_layerDirty = dirty;

    /** @todo: The full repaint can be avoided. The route however has changed
      * and the exact bounding box needs to be recalculated before doing
      * a partly repaint, otherwise we might end up repainting only parts of the route
      */
    // d->m_marbleWidget->repaint( d->m_routeRegion );
    d->m_marbleWidget->update();
}


void MapViewMapLayer::setViewportChanged()
{
    d->m_viewportChanged = true;
}

void MapViewMapLayer::addDocument(GeoDataDocument* pDoc,bool bGOV)
{
	if(bGOV)
	{
		GeoDataFeature* pFeature = static_cast<GeoDataFeature*>(pDoc);
		GeoDataGroundOverlay* pOvl = static_cast<GeoDataGroundOverlay*>(pFeature);
		if(pOvl)
		{
			DocumentRole role = MapDocument;
			GeoDataDocument* pTempDoc = new GeoDataDocument();
			pTempDoc->setDocumentRole(role);
			pTempDoc->setFileName(m_qsFile);
			pTempDoc->append(pFeature);
			d->addDocument(pTempDoc,bGOV);
		}
	}
	else
		d->addDocument(pDoc,bGOV);
}

QString MapViewMapLayer::runtimeTrace() const
{
    return d->m_layerName;
}

void MapViewMapLayer::slotDeleteTreeItem(int type,long id)
{
	//reset all selection members
	if(type==170 || type==171)
	{
	/*	TreeNode* pNode = d->m_treeviewModel->getNodeFromIndexId(id);
		if(pNode==NULL)return;
		if(pNode->ItemType == Layer)*/
	//	if(d->m_pMap!=NULL && d->m_pMap->isVisible())
	//		d->m_marbleWidget->model()->treeModel()->removeDocument(d->m_pMap);

	//	delete d->m_pMap;
		if(d->m_isFeature)
			d->removeDocument();
		d->m_pMap=NULL;
	}
	
}

void MapViewMapLayer::setInputEnabled(bool val)
{
	if(d!=NULL)
		d->m_bInputEnabled = val;
}

// Overriding LayerInterface to paint on top of the route
qreal MapViewMapLayer::zValue() const
{
	return m_zValue;
	//d->m_pMap->setVisualCategory(GeoDataFeature::GeoDataVisualCategory
}

void MapViewMapLayer::setLayerZValue(qreal zValue)
{
	m_zValue = zValue;
	int val = d->m_pMap->zoomLevel();
	d->m_pMap->setZoomLevel((int)zValue);
	d->m_pMap->setPopularity((qint64)zValue);
	d->m_marbleWidget->model()->treeModel()->updateFeature(d->m_pMap);
}

void MapViewMapLayer::updateFeature(qreal left,qreal top,qreal right,qreal bottom)
{
/*	if(d->m_pMap!=NULL)
	{
		if(m_zValue<2)
			m_zValue=2;
		else m_zValue=1;
		d->m_pMap->setZoomLevel((int)m_zValue);
		d->m_marbleWidget->model()->treeModel()->updateFeature(d->m_pMap);
		d->m_marbleWidget->update();
	}
*/	

/*	if(d->m_pMap!=NULL)
	{
		d->m_marbleWidget->model()->treeModel()->removeDocument(d->m_pMap);
	//	m_pMap->setVisible(bVisible);
	//	m_pMap->setZoomLevel((int)q->m_zValue);
		d->m_marbleWidget->model()->treeModel()->addDocument(d->m_pMap);
	}*/

/*	if(d->m_pMap!=NULL)
	{
		d->m_marbleWidget->model()->treeModel()->removeFeature(d->m_pMap,d->m_iFeatureIndex);
		d->m_marbleWidget->model()->treeModel()->addFeature(d->m_marbleWidget->model()->treeModel()->rootDocument(),d->m_pMap);
		addDocument(d->m_pMap);
		GeoDataDocument* pDoc = d->m_pMap;
		d->m_pMap=NULL;
		addDocument(pDoc);
	}*/

	if(d->m_pMap!=NULL && m_pObject!=NULL)
	{
		//GeoDataLatLonAltBox box;
		//const GeoDataOverlay* overlay = dynamic_cast<const GeoDataOverlay*>( d->m_pMap );
		//GeoDataContainer *container =  dynamic_cast<GeoDataContainer*>(d->m_pMap);
		d->m_marbleWidget->model()->treeModel()->index(m_pObject);
	//	GeoDataOverlay* overlay = dynamic_cast<GeoDataOverlay*>( m_pObject );
		GeoDataFeature* feature = dynamic_cast<GeoDataFeature*>( m_pObject );
		GeoDataContainer *container =  dynamic_cast<GeoDataContainer*>(feature);
		const char* chType=feature->nodeType();
	//	if(object==NULL)return;
		QVector<GeoDataFeature*>::Iterator i = container->begin();
		QVector<GeoDataFeature*>::Iterator const end = container->end();
		for (; i != end; ++i ) 
		{
			chType = (*i)->nodeType();
			if( chType == GeoDataTypes::GeoDataFolderType)
			{
				GeoDataContainer *child = static_cast<GeoDataContainer*>( *i );
				QVector<GeoDataFeature*>::Iterator ic = child->begin();
				QVector<GeoDataFeature*>::Iterator const endc = child->end();
				for (; ic != endc; ++ic ) 
				{
					chType = (*ic)->nodeType();
				
					if( chType == GeoDataTypes::GeoDataGroundOverlayType) 
					{
						GeoDataGroundOverlay *overlay =  dynamic_cast<GeoDataGroundOverlay*>(*ic);
						if(overlay!=NULL)
						{
							d->m_goBox.setWest(left,GeoDataCoordinates::Degree);
							d->m_goBox.setEast(right,GeoDataCoordinates::Degree);
							d->m_goBox.setNorth(top,GeoDataCoordinates::Degree);
							d->m_goBox.setSouth(bottom,GeoDataCoordinates::Degree);
							overlay->setIcon(QImage());
							overlay->setLatLonBox(d->m_goBox);
						} 
					}
				}
			}
		}
		//d->m_marbleWidget->update();
		d->m_marbleWidget->model()->treeModel()->updateFeature(d->m_pMap);
		
	/*	const char* type=feature->nodeType();
		GeoDataObject *pObject=feature->parent();
		GeoDataOverlay* overlay = dynamic_cast<GeoDataOverlay*>( pObject );
		//GeoDataOverlay* overlay = (GeoDataOverlay*)feature;
		
		GeoDataGroundOverlay* groundOverlay = static_cast<GeoDataGroundOverlay*>( feature );
		if(groundOverlay!=NULL)
		{
			groundOverlay->setIcon(QImage());
		}*/
	}
}

bool MapViewMapLayer::getLatLonBox(GeoDataLatLonBox& box,qreal& lon,qreal& lat)
{
	box = d->m_goBox;
	lon = d->m_dLongitude;
	lat = d->m_dLatitude;

	return d->m_bGroundOverlay;
}

void MapViewMapLayer::updateDocumentListing(int flag,qreal zval)
{
	if(flag==0)
		d->removeDocument();
	else d->addDocument(zval);

}

bool MapViewMapLayer::pixelToCoord(int xpos,int ypos, double& lon,double& lat)
{
	if(!d->m_bGroundOverlay)return false;
	bool ret = d->pixelToCoord(xpos,ypos,lon,lat);
	return ret;
}

bool MapViewMapLayer::coordToPixel(int& xpos,int& ypos, double lon,double lat)
{
	if(!d->m_bGroundOverlay)return false;
	bool ret = d->coordToPixel(xpos,ypos,lon,lat);
	return ret;
}

} // namespace Marble

#include "moc_MapViewMapLayer.cpp"
