/******************************************************************************************************************
 *	@file			MapViewMapLayer.h
 *	@copyright		2013 Rohde&Schwarz, Munich
 *	@version		$Header: MapViewMapLayer.h   
 *	10 Dezember 2011 16:29:35  enzensbe $
 *
 *	@author			Konrad Enzensberger		e.konrad@mpegcode.com
 *	@responsible	KOnrad Enzensberger
 * 
 *	@language		C++ ANSI V3
 *
 *	@description 	This file defines the MapView dimmer layer.
 *					
 *					
 *****************************************************************************************************************/

#ifndef MARBLE_MAPVIEWMAPLAYER_H
#define MARBLE_MAPVIEWMAPLAYER_H

#include "GeoDataCoordinates.h"
#include "LayerInterface.h"

#include <QtCore/QModelIndex>
#include <QtGui/QItemSelection>

#include "Enum.h"
using namespace GIS;

namespace Marble
{

class MarbleWidget;
class MapViewMapLayerPrivate;
class TreeViewModel;
class GeoDataDocument;
class GeoDataLatLonBox;
class GeoDataObject;

/**
  * @brief A map layer that serves as a view off raster/vector data / MapInfo
  */
class MapViewMapLayer: public QObject, public LayerInterface
{
    Q_OBJECT

public:
    /**
      * @brief Constructor
      * @param widget The marble widget used for geopos <-> screenpos transformations
      *   and repainting of (small) areas. Must not be null
      * @param parent Optional parent widget
      */
    explicit MapViewMapLayer( MarbleWidget *widget, QWidget *parent = 0, qint64 index=0);

    /** Destructor */
    ~MapViewMapLayer();

    /** Reimplemented from LayerInterface. We'll hover above the surface */
    QStringList renderPosition() const;

    /** Reimplemented from LayerInterface. Paints route items and placemarks */
    bool render( GeoPainter *painter, ViewportParams *viewport,
                 const QString &renderPos = "NONE", GeoSceneLayer *layer = 0 );


    /**
      * The route is dirty (needs an update) and should be painted to indicate that
      */
    void setLayerDirty( bool dirty );

	 /**
      * Set the proxy model another QAbstractItemView uses that should share
      * its selection model with us. Needed because this class uses an unfiltered
      * model which has different indices than a filtered one.
      */
    void synchronizeWith( QItemSelectionModel *selection );

	 /**
      * Set the tree model to use. Implicitly removes the placemark model.
      */
    void setModel ( TreeViewModel *model );

	 /**
      * get the Interface ID for TcpIp writing
      */
	long getInterfaceID(void){return m_InterfaceID;}

	/**
      * add the document datat to map layer
      */
	void addDocument(GeoDataDocument* pDoc,bool bGOV=false);

	void setFileName(QString& name){m_qsFile=name;}
	QString getFileName(void){return m_qsFile;}

	/**
      * input enabled flag - if true, reject mouse move over object
      */
	void setInputEnabled(bool val);

	// Overriding LayerInterface to paint on top of other layers
    virtual qreal zValue() const;

	/**
      * set the layer zvalue
      */
	void setLayerZValue(qreal zValue);

	/**
      * get the GroundOverlay LatlonBox
      */
	bool getLatLonBox(GeoDataLatLonBox& box,qreal& lon,qreal& lat);

	/**
      * update document sort
      */
	void updateDocumentListing(int flag,qreal zval);

	/**
	  * convert pixel pos to geo coordinates - check if pixel position within GroundOverlay
	  */
	bool pixelToCoord(int xpos,int ypos, double& lon,double& lat);

	/**
	  * convert geo coordinates to pixel pos - check if pixel position within GroundOverlay
	  */
	bool coordToPixel(int& xpos,int& ypos, double lon,double lat);

	/**
	*	used to display layername in runtime trace 
	*/
	virtual QString runtimeTrace() const;

	/**
	*	update new ground overlay image 
	*/
	void updateFeature(qreal left,qreal top,qreal right,qreal bottom);


protected:
    /** Overriding QWidget, used to make the layer interactive */
    bool eventFilter( QObject *obj, QEvent *event );

private Q_SLOTS:

	 /**
      * The viewport has changed, recalculate positions accordingly
      */
    void setViewportChanged();

public Q_SLOTS:
	/**
      * free selection index if item is deleted by tree
      */
	void slotDeleteTreeItem(int,long);

	void slotRemoved( GeoDataObject *object );
    void slotAdded( GeoDataObject *object );

Q_SIGNALS:
	/**
	  * send mouse pos 
	*/
	void signalNotifyMouseAction(QPoint&,int,long intID);

	/**
	  * send mouse and image pos 
	*/
	void signalNotifyImageMouseAction(QPoint&,QPoint&,int,long intID);

	/**
      * send mouse pos if mouse hovers object
      */
	void mouseMoveGeoPosition( QString );

private:
    MapViewMapLayerPrivate *const d;
    friend class MapViewMapLayerPrivate;
	GeoDataObject *m_pObject;
	long	m_InterfaceID;
	qreal	m_zValue;
	QString m_qsFile;

};

} // namespace Marble

#endif
