/*******************************************************************************
* @file			MapViewShadingLayer.cpp
* @copyright	2011-2013 Rohde&Schwarz, Munich
* @version		$Header: MapViewShadingLayer.h
*
* @author		Konrad Enzensberger		e.konrad@mpegcode.com
* @responsible	KOnrad Enzensberger
* 
* @language		C++ ANSI V3
*
* @history
*
*
********************************************************************************
*******************************************************************************/

/* INCLUDE FILES **************************************************************/

#include "MapViewShadingLayer.h"

#include "GeoDataCoordinates.h" 
#include "GeoDataLineString.h"
#include "GeoSceneDocument.h"
#include "GeoSceneHead.h"
#include "GeoPainter.h"
#include "MarbleWidget.h"
#include "MarbleMap.h"
#include "MarbleModel.h"
#include "TreeViewModel.h"
#include "GeoDataTreeModel.h"
#include "MarbleWidgetPopupMenu.h"
#include "ViewportParams.h"

#include "GeoDataDocument.h"
#include "GeoDataFolder.h"
#include "GeoDataPlacemark.h"
#include "GeoDataMultiGeometry.h"
#include "GeoDataExtendedData.h"
#include "GeoDataGroundOverlay.h"
#include "GeoDataStyle.h"
#include "GeoDataStyleMap.h"
#include "GeoDataLineStyle.h"
#include "GeoDataFeature.h"
#include "GeoWriter.h"

#include <QtCore/QMap>
#include <QtGui/QIcon>
#include <QtGui/QKeyEvent>
#include <QtGui/QMouseEvent>
#include <QtGui/QMenu>
#include <QtGui/QPixmap>
#include <QtGui/QImage>
#include <QtGui/QItemSelectionModel>
#include <QImageWriter>
#include <QFileInfo>

namespace Marble
{

class MapViewShadingLayerPrivate
{
    template<class T>
    struct PaintRegion {
        T index;
        QRegion region;

        PaintRegion( const T &index_, const QRegion &region_ ) :
                index( index_ ), region( region_ )
        {
            // nothing to do
        }
    };

    typedef PaintRegion<QModelIndex> ModelRegion;
    typedef PaintRegion<int> RequestRegion;

public:
    MapViewShadingLayer *q;

	qint64 m_indexID;

	bool m_viewportChanged;

    MarbleWidget*			m_marbleWidget;
	QList<ModelRegion>		m_shadingRegion;	//region from polygones

 //   QPixmap m_targetPixmap;
	QMap<long,QImage>	m_qmImage;
	QMap<long,GeoDataDocument*> m_qmImageDoc;
	GeoDataDocument*	m_pMap;

    QRect m_dirtyRect;
	int		m_iDimmer;

	QItemSelectionModel*	m_selectionModel;
    TreeViewModel*			m_treeviewModel;
	TreeNode*				m_pNode;
	QModelIndex				m_selIdx;
	bool					m_bGrid;

    bool m_layerDirty;

	QSize m_pixmapSize;
	QColor	m_lineColor;
	QColor	m_fillColor;
	QColor	m_fontColor;
	QColor	m_fontBackColor;
	int		m_lineWidth;

	bool	m_bEdit;
	bool	m_bDelete;
	bool	m_bMoveObj;
	bool	m_bDeleteObj;
	bool	m_bInputEnabled;	//if true, do not take care obout mouse move over objects

	//holds the menu items sent by TcpIp
	QActionGroup*			m_pActionGroup;
	MarbleWidgetPopupMenu	*m_contextMenu;

    /** Constructor */
    explicit MapViewShadingLayerPrivate( MapViewShadingLayer *parent, MarbleWidget *widget, qint64 index );

    /** Paint waypoint polygon */
    inline void renderLayer( GeoPainter *painter );

	/** Insert via points or emit position signal, if appropriate */
    inline bool handleMouseButtonRelease( QMouseEvent *e );

    /** Select route instructions points, start dragging trip points */
    inline bool handleMouseButtonPress( QMouseEvent *e );

    /** Dragging trip points, route polygon hovering */
    inline bool handleMouseMove( QMouseEvent *e );

    /** Escape to stop selecting points */
    inline bool handleKeyEvent( QKeyEvent *e );

	 /** True if the given point (screen coordinates) is among the route instruction points */
    inline bool isInfoPoint( const QPoint &point );

	 /** Show a context menu at the specified position */
    void showContextMenu( const QPoint &position );

	/** add menue entries sent by TcpIp */
	inline void addContextMenu(QString& qsMenu,int iVal);
	/** delete menue entries sent by TcpIp */
	inline void delContextMenu(void);

	/** export layer objects to kml */
	inline void export2kml(QString&);
	/** export layer objects to kml */
	inline void exportOverlay2kml(GeoDataDocument* document,GeoDataFolder* folder);
	/** add icon to kml */
	inline void addIconToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);
	/** add label to kml */
	inline void addLabelToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);
	/** add rect to kml */
	inline void addRectToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);
	/** add polygone to kml */
	inline void addPolygoneToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);
	/** add polyline to kml */
	inline void addPolyToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);
	/** add ellipse to kml */
	inline void addEllipseToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);
	/** add azimuth to kml */
	inline void addAzimuthToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);
	/** add plot object to kml */
	inline void addPlotToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);
	/** add plotEx object to kml */
	inline void addPlotExToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);
	/** add colored area to kml */
	inline void addGroundOverlayToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);


	inline bool calculateImageCoord(QImage& img,const GeoDataCoordinates& lt,const GeoDataCoordinates& rb,const int& width,const int& heigth,const int& iGridX,const int& iGridY,const int& iID);

	inline bool addDocument(long id,bool state);


};

MapViewShadingLayerPrivate::MapViewShadingLayerPrivate( MapViewShadingLayer *parent, MarbleWidget *widget, qint64 index ) :
        q( parent ),  m_marbleWidget( widget ),
        m_treeviewModel( 0 ),m_pNode(0), m_selectionModel( 0 ),
        m_layerDirty( false ), m_pixmapSize( 22, 22 ),m_pMap(NULL),
		m_indexID(index), m_viewportChanged(true),m_pActionGroup(NULL),m_bGrid(false),m_bInputEnabled(false)
{
	m_contextMenu = new MarbleWidgetPopupMenu( m_marbleWidget, m_marbleWidget->model() );
	m_iDimmer=0;
	m_lineColor = QColor( 0,0,0);//black
	m_fillColor = QColor( 255,0, 0, 0 );//red - transparent
	m_fontColor = QColor( 0,0, 0);//black
	m_fontBackColor = QColor( 255,255,255);//white
	m_lineWidth=2;

	m_bEdit=true;
	m_bDelete=true;
	m_bMoveObj=true;
	m_bDeleteObj=true;
}

void MapViewShadingLayerPrivate::addContextMenu(QString& qsMenu,int iVal)
{
	if(m_pActionGroup==NULL)
	{
		m_pActionGroup = new QActionGroup(m_marbleWidget);
		if(m_pActionGroup==NULL)
			return;
		QObject::connect( m_pActionGroup, SIGNAL( triggered(QAction*) ), q, SLOT( slotLayerContextMenu(QAction*) ) );
		m_contextMenu->addAction(Qt::RightButton,m_contextMenu->getRmbMenu()->addSeparator());
	}

	if(qsMenu != "MapView")
	{
		QAction* lcmAction = new QAction(qsMenu, q );
		m_pActionGroup->addAction(lcmAction);
		lcmAction->setData(iVal);
		lcmAction->setEnabled(iVal<100);//menu id > 100 is disabled
		//QObject::connect( m_lcmAction, SIGNAL( triggered(QAction*) ), q, SLOT( slotLayerContextMenu(QAction*) ) );
		m_contextMenu->addAction( Qt::RightButton, lcmAction );
	}
	else //only menu set by TcpIp is used
	{
		if(m_contextMenu!=NULL)
		{
			QMenu* qMenu = m_contextMenu->getRmbMenu();
			if(qMenu!=NULL)
			{
				
			}
		}
	}

}

void MapViewShadingLayerPrivate::delContextMenu()
{
	if(m_pActionGroup!=NULL)
	{
		if(m_contextMenu!=NULL)
		{
			QMenu* qMenu = m_contextMenu->getRmbMenu();
			if(qMenu!=NULL)
			{
				foreach( QAction *action, qMenu->actions() ) 
				{
					QVariant qVar = action->data();
					int iVal = qVar.toInt();
					if ( (action->text() != QObject::tr( "&Remove" )) && (action->text() != QObject::tr( "&Insert" )) && (action->text() != QObject::tr( "&Delete" )) ) 
					{
						qMenu->removeAction( action );
					}
				}
			}
		}
		QObject::disconnect(m_pActionGroup,SIGNAL( triggered(QAction*) ), q, SLOT( slotLayerContextMenu(QAction*) ) );
		delete m_pActionGroup;
		m_pActionGroup=NULL;
	}
}

void MapViewShadingLayerPrivate::showContextMenu( const QPoint &pos )
{
//	m_bIsPoly=false;
    m_contextMenu->showRmbMenu( pos.x(), pos.y() );
}

void MapViewShadingLayerPrivate::export2kml(QString& filename)
{
	if(m_indexID==0)return;

	TreeNode* pNode = m_treeviewModel->getNodeFromIndexId(m_indexID);
	if(pNode==NULL) return;
	if(pNode->ItemType != Layer) return;

	QString qsOverlay = pNode->parent->m_qsName;
	QString qsLayer   = pNode->m_qsName;
	int iGroupCount   = pNode->children.count();

}

void MapViewShadingLayerPrivate::exportOverlay2kml(GeoDataDocument* document,GeoDataFolder* folder)
{
	if(m_indexID==0)return;
	if(document==NULL)return;
	if(folder==NULL)return;

	TreeNode* pNode = m_treeviewModel->getNodeFromIndexId(m_indexID);
	if(pNode==NULL) return;
	if(pNode->ItemType != Layer) return;

	QString qsOverlay = pNode->parent->m_qsName;
	QString qsLayer   = pNode->m_qsName;
	int iGroupCount   = pNode->children.count();

	//document->setName(qsOverlay);

	TreeNode* pGroup=NULL;
	TreeNode* pChild=NULL;
	int iChildCount=0;

	for(int iG=0;iG<iGroupCount;iG++)//count the group
	{
		pGroup = pNode->children.at(iG);
		if(pGroup!=NULL)
		{
			iChildCount = pGroup->children.count();
			//for(int iC=0;iC<iChildCount;iC++)
			if(iChildCount==1)//group with one child
			{
				pChild =  pGroup->children.at(0);
				if(pChild != NULL)
				{
					switch(pChild->ItemType)
					{
						case Icon:	
							 addIconToKml(pGroup,pChild,document,folder);
							 break;
					    case Text:
						case Desc:
							 addLabelToKml(pGroup,pChild,document,folder);
							 break;
						case Rect:	
							addRectToKml(pGroup,pChild,document,folder);
							break;
						case Polygone:	
							addPolygoneToKml(pGroup,pChild,document,folder);
							break;
						case Poly:	
							addPolyToKml(pGroup,pChild,document,folder);
							break;
						case Elipse:	
							addEllipseToKml(pGroup,pChild,document,folder);
							break;
						case Azimuth:	
							addAzimuthToKml(pGroup,pChild,document,folder);
							break;
						case Plot:	
							addPlotToKml(pGroup,pChild,document,folder);
							break;
						case PlotEx:	
							addPlotExToKml(pGroup,pChild,document,folder);
							break;
						case Shading:
							addGroundOverlayToKml(pGroup,pChild,document,folder);
							break;
					}

				}//if one Child
				
			}//
			//else	//MultiGeometry
			//	addMultiGeometryToKml(pGroup,document,folder);

		}//if pGroup

	}//for GroupCount

}

void MapViewShadingLayerPrivate::addGroundOverlayToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );
	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	QString name = document->fileName();
	QFileInfo info(name);
	name = info.absolutePath();
	name += "/";
	name += pGroup->m_qsName;
	name += ".png";

	qint64 iID =(qint64)pChild->m_lId;
	QModelIndex idxChild = m_treeviewModel->getIndexFromIndexId(iID);
	if(!idxChild.isValid())return;

	QVariant qVar = m_treeviewModel->data(idxChild,TreeViewModel::InfoRole);
	int idGrid = qVar.toInt();

	if(m_qmImage.contains((long)idGrid ) )
	{
		QImage qImage = m_qmImage.value((long)idGrid);
		if(qImage.isNull())
			return;

		QImageWriter qImgw;
		qImgw.setFormat("png"); 
		qImgw.setFileName(name);
		qImgw.write(qImage);

		GeoDataGroundOverlay* overlay = new GeoDataGroundOverlay();
		if(overlay!=NULL)
		{
			overlay->setIconFile(name);

			GeoDataLatLonBox box = pChild->m_poly.latLonAltBox();
			overlay->setLatLonBox(box);
		
			folder->append( overlay );
		}
	}

	folder->append( placemark );
}

void MapViewShadingLayerPrivate::addIconToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pGroup->m_qsName).arg(pChild->m_lId);

	lineStyle.setColor(m_fillColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );
	//style.setStyleId( m_lineColor.name().replace( '#', 'f' ) );
	style.setStyleId(styleID);

	QPixmap qPix = m_treeviewModel->pixmap(pChild->m_qsImage);
	if(!qPix.isNull())
	{
		//to export ToolTip as <description>
		if(!pGroup->m_qsToolTip.isEmpty())
			placemark->setDescription(pGroup->m_qsToolTip);

		iconStyle.setIcon(qPix.toImage());
		iconStyle.setScale(1.0);
		iconStyle.setIconPath(pChild->m_qsImage);
	
		style.setIconStyle(iconStyle);
		document->addStyle( style );
		placemark->setStyleUrl(  QString( "#" ).append(styleID ));

		GeoDataCoordinates pos = pGroup->m_position;
		placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

		folder->append( placemark );
	}

}

void MapViewShadingLayerPrivate::addLabelToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	QString qsLabel; 
	if(pChild->ItemType==Desc)
	{
		qsLabel = pChild->m_qsDesc;
		if(!qsLabel.isEmpty())
		{
			placemark->setName( qsLabel );
			//to export ToolTip as <description>
			if(!pGroup->m_qsToolTip.isEmpty())
				placemark->setDescription(pGroup->m_qsToolTip);

			GeoDataCoordinates pos = pGroup->m_position;
			placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

			folder->append( placemark );
		}
	}
	else if(pChild->ItemType==Text)
	{
		qsLabel = pChild->m_qsName;
		if(!qsLabel.isEmpty())
		{
			placemark->setName( qsLabel );
			//to export ToolTip as <description>
			if(!pGroup->m_qsToolTip.isEmpty())
				placemark->setDescription(pGroup->m_qsToolTip);

			GeoDataCoordinates pos = pGroup->m_position;
			placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

			folder->append( placemark );
		}
	}

}
void MapViewShadingLayerPrivate::addRectToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
	GeoDataPolyStyle	polyStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	QColor col;
	long colorRef=0;
	int red=0;
	int green=0;
	int blue=0;

	if(pChild->m_lLineCol != -1)
	{
		colorRef = pChild->m_lLineCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		lineStyle.setColor(col);
	}
	else lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );

	if(pChild->m_lFillCol != -1)
	{
		colorRef = pChild->m_lFillCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		if(pChild->m_iTrans != -1)
			col.setAlpha(pChild->m_iTrans);
		else col.setAlpha(m_fillColor.alpha());
		polyStyle.setColor(col);
	}
	else polyStyle.setColor(m_fillColor);
	style.setPolyStyle(polyStyle);

	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) ); 

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLinearRing gdLS; 
	GeoDataCoordinates pos2;
	
	pos  = pChild->m_poly.at(0);
	pos2 = pChild->m_poly.at(1);

	gdLS << pos;
	pos.setLongitude(pos2.longitude());
	gdLS << pos;
	pos.setLatitude(pos2.latitude());
	gdLS << pos;
	pos  = pChild->m_poly.at(0);
	pos.setLatitude(pos2.latitude());
	gdLS << pos;
	pos  = pChild->m_poly.at(0);
	gdLS << pos;
	placemark->setGeometry( new GeoDataLinearRing( gdLS) );

	folder->append( placemark );

}
void MapViewShadingLayerPrivate::addPolygoneToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
	GeoDataPolyStyle	polyStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	QColor col;
	long colorRef=0;
	int red=0;
	int green=0;
	int blue=0;
	int alpha=0;

	if(pChild->m_lLineCol != -1)
	{
		colorRef = pChild->m_lLineCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		lineStyle.setColor(col);
	}
	else lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );

	if(pChild->m_lFillCol != -1)
	{
		colorRef = pChild->m_lFillCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		alpha     = ( colorRef>>24 ) & 255;
		col = QColor( red, green, blue );
		if(pChild->m_iTrans != -1)
			col.setAlpha(pChild->m_iTrans);
		else col.setAlpha(alpha);
		polyStyle.setColor(col);
	}
	else polyStyle.setColor(m_fillColor);
	style.setPolyStyle(polyStyle);

	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLinearRing gdLS;
	gdLS << pChild->m_poly;
	gdLS << pChild->m_poly.at(0);
	placemark->setGeometry( new GeoDataLinearRing( gdLS) );

	folder->append( placemark );

}
void MapViewShadingLayerPrivate::addPolyToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	QColor col;
	long colorRef=0;
	int red=0;
	int green=0;
	int blue=0;

	if(pChild->m_lLineCol != -1)
	{
		colorRef = pChild->m_lLineCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		lineStyle.setColor(col);
	}
	else lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );
	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLineString gdLS;
	gdLS << pChild->m_poly;
	//gdLS << pNode->m_poly.at(0);
	placemark->setGeometry( new GeoDataLineString( gdLS) );

	folder->append( placemark );

}
void MapViewShadingLayerPrivate::addEllipseToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
	GeoDataPolyStyle	polyStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	QColor col;
	long colorRef=0;
	int red=0;
	int green=0;
	int blue=0;

	if(pChild->m_lLineCol != -1)
	{
		colorRef = pChild->m_lLineCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		lineStyle.setColor(col);
	}
	else lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );

	if(pChild->m_lFillCol != -1)
	{
		colorRef = pChild->m_lFillCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		if(pChild->m_iTrans != -1)
			col.setAlpha(pChild->m_iTrans);
		else col.setAlpha(m_fillColor.alpha());
		polyStyle.setColor(col);
	}
	else polyStyle.setColor(m_fillColor);
	style.setPolyStyle(polyStyle);

	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLinearRing gdLS;
	gdLS << pChild->m_poly;
	gdLS << pChild->m_poly.at(0);
	placemark->setGeometry( new GeoDataLinearRing( gdLS) );

	folder->append( placemark );

}
void MapViewShadingLayerPrivate::addAzimuthToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	QColor col;
	long colorRef=0;
	int red=0;
	int green=0;
	int blue=0;

	if(pChild->m_lLineCol != -1)
	{
		colorRef = pChild->m_lLineCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		lineStyle.setColor(col);
	}
	else lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );
	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLineString gdLS;
	gdLS << pChild->m_poly;
	//gdLS << pNode->m_poly.at(0);
	placemark->setGeometry( new GeoDataLineString( gdLS) );

	folder->append( placemark );

}

void MapViewShadingLayerPrivate::addPlotToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );
	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLineString gdLS;
	gdLS << pChild->m_poly;
	gdLS << pChild->m_poly.at(0);
	placemark->setGeometry( new GeoDataLinearRing( gdLS) );

	folder->append( placemark );

}

void MapViewShadingLayerPrivate::addPlotExToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );
	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLineString gdLS;
	gdLS << pChild->m_poly;
	gdLS << pChild->m_poly.at(0);
	placemark->setGeometry( new GeoDataLinearRing( gdLS) );

	folder->append( placemark );

}

void MapViewShadingLayerPrivate::renderLayer( GeoPainter *painter )
{
  //  GeoDataLineString waypoints;

	QModelIndex idxParent = m_treeviewModel->getIndexFromIndexId(m_indexID);

	QModelIndex idxOverlay = m_treeviewModel->parent(idxParent);
	QVariant qVar = m_treeviewModel->data(idxOverlay,Qt::CheckStateRole);

	bool bVisible = qVar.toBool();
	if(!bVisible)return;
	//Overlay disabled ?
	if(!bVisible)
	{
		QMap<long,GeoDataDocument*>::const_iterator docIter = m_qmImageDoc.constBegin();
		for ( ; docIter != m_qmImageDoc.constEnd(); ++docIter )
		{
			GeoDataDocument* pDoc = docIter.value();
			if(pDoc!=NULL&&pDoc->isVisible() )
			{
				pDoc->setVisible(bVisible);
				pDoc->setZoomLevel((int)q->m_zValue);
				m_marbleWidget->model()->treeModel()->updateFeature(pDoc);
			}
		}

		return;
	}
/*	else if(!bVisible && m_pMap->isVisible())
	{
		m_pMap->setVisible(bVisible);
		m_marbleWidget->model()->treeModel()->updateFeature(m_pMap);
	}*/
	
	//Layeray disabled ?
	qVar = m_treeviewModel->data(idxParent,Qt::CheckStateRole);
	bVisible = qVar.toBool();
	if(!bVisible)return;
	if(!bVisible)
	{
		QMap<long,GeoDataDocument*>::const_iterator docIter = m_qmImageDoc.constBegin();
		for ( ; docIter != m_qmImageDoc.constEnd(); ++docIter )
		{
			GeoDataDocument* pDoc = docIter.value();
			if(pDoc!=NULL&&pDoc->isVisible() )
			{
				pDoc->setVisible(bVisible);
				pDoc->setZoomLevel((int)q->m_zValue);
				m_marbleWidget->model()->treeModel()->updateFeature(pDoc);
			}
		}

		return;
	}

	qVar = m_treeviewModel->data(idxParent,Qt::UserRole);
	long colorRef = (long)qVar.toLongLong();
	int red      = colorRef & 255;
	int green    = ( colorRef>>8 ) & 255;
	int blue     = ( colorRef>>16 ) & 255;
	QColor color = QColor( red, green, blue );
	
	if(m_viewportChanged)
	{
		m_shadingRegion.clear();
	}
	

	//group item count
	int iRowCount = m_treeviewModel->rowCount(idxParent);
	int iReg=0;

	//if(iRowCount==0)
	//	m_selIdx=QModelIndex();

    for ( int i = 0; i < iRowCount; ++i ) 
	{
		//Group
        QModelIndex index = m_treeviewModel->index( i, 0,idxParent );

		if ( index.isValid() )
		{
			qVar = m_treeviewModel->data(index,Qt::CheckStateRole);
			bVisible=qVar.toBool();
			if(0)//!bVisible)//disable all child grids
			{
				int iChildCount = m_treeviewModel->rowCount(index);
				if(iChildCount>0)//m_treeviewModel->rowCount(index)>0)
				{
					for ( int ic = 0; ic < iChildCount; ++ic ) 
					{
						QModelIndex idxChild = m_treeviewModel->index( ic, 0,index );
						long groupID = (long)idxChild.internalId();
						if(m_qmImageDoc.contains(groupID) )
						{
							GeoDataDocument* pDoc = m_qmImageDoc.value(groupID);
							if(pDoc!=NULL&&pDoc->isVisible() )
							{
								pDoc->setVisible(bVisible);
								pDoc->setZoomLevel((int)q->m_zValue);
								m_marbleWidget->model()->treeModel()->updateFeature(pDoc);
							}
						}
					}
				}
			}
			else if(bVisible)
			{
				//group child count
				int iChildCount = m_treeviewModel->rowCount(index);
				GeoDataCoordinates pos = qVariantValue<GeoDataCoordinates>( index.data( TreeViewModel::CoordinateRole ) );
				GIS::TreeViewItemType type = (GIS::TreeViewItemType) qVariantValue<unsigned int>( index.data( TreeViewModel::TypeRole ) );
				
				if(iChildCount>0)//m_treeviewModel->rowCount(index)>0)
				{
					for ( int ic = 0; ic < iChildCount; ++ic ) 
					{
						QModelIndex idxChild = m_treeviewModel->index( ic, 0,index );
						if(idxChild.isValid())
						{
							qVar = m_treeviewModel->data(idxChild,Qt::CheckStateRole);
							bVisible = qVar.toBool();
							if(0)//!bVisible)
							{
								long groupID = (long)idxChild.internalId();
								if(m_qmImageDoc.contains(groupID) )
								{
									GeoDataDocument* pDoc = m_qmImageDoc.value(groupID);
									if(pDoc!=NULL&&pDoc->isVisible() )
									{
										pDoc->setVisible(bVisible);
										pDoc->setZoomLevel((int)q->m_zValue);
										m_marbleWidget->model()->treeModel()->updateFeature(pDoc);
									}
								}

							}
							else if(bVisible)
							{
								GIS::TreeViewItemType cType = (GIS::TreeViewItemType) qVariantValue<unsigned int>( idxChild.data( TreeViewModel::TypeRole ) );
								
								if (cType == GIS::Shading )
								{
									/*qVar = m_treeviewModel->data(idxChild,TreeViewModel::DataRole);//Qt::UserRole);
									colorRef = (long)qVar.toLongLong();
									red      = colorRef & 255;
									green    = ( colorRef>>8 ) & 255;
									blue     = ( colorRef>>16 ) & 255;*/
									color = QColor( 0, 0, 0 );// = QColormap::instance().colorAt(ref));
									//painter->setPen( QColor( 255,255,0, 255 ) );
									painter->setPen(color);
									painter->setBrush( QColor( 99, 198, 99, 80 ) );

									GeoDataCoordinates pos;
									GeoDataLineString* gdl = m_treeviewModel->getPoly(idxChild);
									GeoDataLinearRing  ring;
									for(int i=0;i<gdl->size()-1;i++)
									{
										pos = gdl->at(i);
										ring << pos;
									}
									//if(!qsImage.isEmpty())
									//painter->drawPolyline(*gdl);

									qVar = m_treeviewModel->data(idxChild,TreeViewModel::OffXRole);
									int iXGrid = qVar.toInt();
									qVar = m_treeviewModel->data(idxChild,TreeViewModel::OffYRole);
									int iYGrid = qVar.toInt();
									qVar = m_treeviewModel->data(idxChild,TreeViewModel::InfoRole);
									int iID = qVar.toInt();

									if(m_qmImage.contains((long)iID) )
									{
										QImage qImage = m_qmImage.value((long)iID);
										if(!qImage.isNull())
										{
											qreal x1=1111.0,y1=1111.0,x2=1111.0,y2=1111.0;
											pos = gdl->at(0);
											qreal left = pos.longitude(GeoDataCoordinates::Degree);
											m_marbleWidget->screenCoordinates(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),x1,y1);
											pos = gdl->at(2);
											qreal right = pos.longitude(GeoDataCoordinates::Degree);
											m_marbleWidget->screenCoordinates(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),x2,y2);
											QRect rect(x1,y1,x2-x1,y2-y1);

											if(m_viewportChanged || m_shadingRegion.isEmpty())
											{
												//regionFromPolygon failes if greater than 180
												if((right-left)<=180)
												{
													QRegion region = painter->regionFromPolygon(ring,Qt::WindingFill,0);
													if(!region.isEmpty())
													{
														m_shadingRegion.push_front( ModelRegion( idxChild, region ) );
													}
												}
												else
												{
													GeoDataLinearRing reg1;
													pos = gdl->at(0);
													left = pos.longitude(GeoDataCoordinates::Degree);
													reg1<<pos;
													pos = gdl->at(1);
													right = pos.longitude(GeoDataCoordinates::Degree);
													pos.setLongitude(left+180.0,GeoDataCoordinates::Degree);
													reg1<<pos;
													pos = gdl->at(2);
													pos.setLongitude(left+180.0,GeoDataCoordinates::Degree);
													reg1<<pos;
													pos = gdl->at(3);
													reg1<<pos;
													QRegion region = painter->regionFromPolygon(reg1,Qt::WindingFill,0);
													if(!region.isEmpty())
													{
														m_shadingRegion.push_front( ModelRegion( idxChild, region ) );
													}
													reg1.clear();
													pos = gdl->at(0);
													left = pos.longitude(GeoDataCoordinates::Degree);
													pos.setLongitude(left+180.0,GeoDataCoordinates::Degree);
													reg1<<pos;
													pos = gdl->at(1);
													right = pos.longitude(GeoDataCoordinates::Degree);
													reg1<<pos;
													pos = gdl->at(2);
													reg1<<pos;
													pos = gdl->at(3);
													pos.setLongitude(left+180.0,GeoDataCoordinates::Degree);
													reg1<<pos;
													region = painter->regionFromPolygon(reg1,Qt::WindingFill,0);
													if(!region.isEmpty())
													{
														m_shadingRegion.push_front( ModelRegion( idxChild, region ) );
													}
												}
											}

											//enz - respect heading
											if(m_marbleWidget->heading()!=0) 
											{
												//qreal x1(0.0), x2(0.0), y1( 0.0 ), y2( 0.0 );
												//m_marbleWidget->screenCoordinates( p()->m_latLonAltBox.west(), p()->m_latLonAltBox.north(), x1, y1 );
												//m_marbleWidget->screenCoordinates( p()->m_latLonAltBox.east(), p()->m_latLonAltBox.south(), x2, y2 );
												//painter->setCompositionMode(QPainter::RasterOp_SourceAndDestination);

												QRect const screen( QPoint( 0, 0), m_marbleWidget->viewport()->size() );
												QRect const position( x1, y1, x2-x1, y2-y1 );
												if ( !(screen & position).isEmpty() ) 
												{
													painter->save();

													painter->translate( m_marbleWidget->viewport()->width()/2, m_marbleWidget->viewport()->height()/2 ); 
													painter->rotate( m_marbleWidget->viewport()->heading() * RAD2DEG ); 
													painter->translate( -m_marbleWidget->viewport()->width()/2, -m_marbleWidget->viewport()->height()/2 );

													painter->drawImage(position,qImage);
													painter->restore();
												}
										
											}
											else if(0)//m_marbleWidget->heading()!=0)
											{
												qreal width = rect.width()/2;
												qreal height = rect.height()/2;
												width = m_marbleWidget->viewport()->width()/2;
												height = m_marbleWidget->viewport()->height()/2;
												painter->translate( width, height );
												painter->rotate( m_marbleWidget->heading()); 
												painter->translate( -width,-height);


										/*		QTransform rot;
												rot.rotate(m_marbleWidget->heading());
												qImage = qImage.transformed(rot, Qt::SmoothTransformation);*/
												painter->drawImage(rect,qImage);
												painter->resetTransform();
												painter->restore();
											}
											else
												painter->drawImage(rect,qImage);
										}

									/*	QImage qImg(rect.width(),rect.height(),QImage::Format_ARGB32);
										qImg.fill(255);
										if(calculateImageCoord(qImg,gdl->at(0),gdl->at(2),rect.width(),rect.height(),iDirection,iID) )
										{
											//QRect source(0,0,iDirection,iDirection);

											//QImage qImage = m_qmImage.value((long)iID);

											painter->drawImage(rect,qImg);
										}*/
									}


								}//cType
							}//check item state
						}
					}//index valid
				}//for

			}//child count
				
		}//check group state
    }

	m_viewportChanged=false;
}

bool MapViewShadingLayerPrivate::calculateImageCoord(QImage& qImg,const GeoDataCoordinates& lt,const GeoDataCoordinates& rb,const int& width,const int& height,const int& iGridX,const int& iGridY,const int& iID)
{
	bool ret=false;

	QImage qImage = m_qmImage.value((long)iID);
	if(qImage.isNull())return ret;

	if(qImage.width()>=width && qImage.height()>=height)
	{
		qImg = qImage.copy();
		return true;
	}

	QRgb rgb;

	qreal dMinX = lt.longitude(GeoDataCoordinates::Degree);
	qreal dMaxY = lt.latitude(GeoDataCoordinates::Degree);
	qreal dMaxX = rb.longitude(GeoDataCoordinates::Degree);
	qreal dMinY = rb.latitude(GeoDataCoordinates::Degree);

	qreal dX = (rb.longitude(GeoDataCoordinates::Degree)-lt.longitude(GeoDataCoordinates::Degree))/iGridX;
	qreal dY = (lt.latitude(GeoDataCoordinates::Degree)-rb.latitude(GeoDataCoordinates::Degree))/iGridY;
	qreal dLon =0;
	qreal dLat =0;

	qreal x1=0,y1=0,x2=0,y2=0;
	int iCount = iGridX*iGridY;
	int iX=0;
	int iY=0;
	int inX=0;
	int inY=0;

	m_marbleWidget->screenCoordinates(dMinX,dMaxY,x1,y1);
	m_marbleWidget->screenCoordinates(dMaxX,dMinY,x2,y2);

	inX = (int)(x1+0.5);
	inY = (int)(y1+0.5);

	int ixpos1=0;
	int ixpos2=0;
	int iypos1=0;
	int iypos2=0;
	for(int i=0;i<iCount;i++)
	{
		dLon = dMinX + (iX*dX); 
		dLat = dMaxY - (iY*dY); 
		m_marbleWidget->screenCoordinates(dLon,dLat,x1,y1);
		m_marbleWidget->screenCoordinates(dLon+dX,dLat-dY,x2,y2);

		ixpos1 = (int)(x1+0.5);
		ixpos2 = (int)(x2+0.5);
		iypos1 = (int)(y1+0.5);
		iypos2 = (int)(y2+0.5);

		rgb = qImage.pixel(iX,iY);
		for(int y=iypos1;y<=iypos2;y++)
		{
			for(int x=ixpos1;x<=ixpos2;x++)
			{
				qImg.setPixel(x-inX,y-inY,rgb);
			//	inX++;
			/*	if(x==width)
				{
					x=0;
					y++;
				}*/
				//if(inY==height)
				//	inY=0;

			}
		}

		iX++;
		if(iX==iGridX)
		{
			iX=0;
			iY++;
		}
		if(iY==iGridY)
			iY=0;
	}

//	m_qmImage.remove((long)iID);
//	m_qmImage.insert((long)iID,qImg);
	ret=true;

	return ret;

}

/** add a document to the map, created from new image */
bool MapViewShadingLayerPrivate::addDocument(long id,bool state)
{
	if(m_treeviewModel==NULL)return false;

	if(state)//add the GeoDataGroundOverlay
	{
		GeoDataGroundOverlay* pOverlay = new GeoDataGroundOverlay();
		if(pOverlay==NULL)return false;
		QImage qImage = m_qmImage.value(id);
		if(qImage.isNull())return false;

		qint64 qID = (qint64)id;
		TreeNode* pNode = m_treeviewModel->getNodeFromIndexId(qID);
		if(pNode==NULL)return false;

		if(pNode->m_poly.size()<3)return false;
		GeoDataCoordinates cor = pNode->m_poly.at(0);
		GeoDataLatLonBox box;
		box.setWest(cor.longitude());
		box.setNorth(cor.latitude());
		cor = pNode->m_poly.at(2);
		box.setEast(cor.longitude());
		box.setSouth(cor.latitude());

		pOverlay->setIcon(qImage);
		pOverlay->setLatLonBox(box);

		GeoDataDocument* pDoc = new GeoDataDocument();
		if(pDoc==NULL) return false;

		m_pMap = pDoc;
		m_pMap->setVisible(true);
		m_pMap->setZoomLevel((int)q->m_zValue);

		pDoc->append(pOverlay);

		m_qmImageDoc.insert(id,pDoc);
		m_marbleWidget->model()->treeModel()->addDocument(pDoc);

	}
	else
	{
		GeoDataDocument* pDoc = m_qmImageDoc.value(id);
		if(pDoc!=NULL)
		{
			m_qmImageDoc.remove(id);
			m_marbleWidget->model()->treeModel()->removeDocument(pDoc);
		}
	//	m_iFeatureIndex=-1;
	}

}

/** Insert via points or emit position signal, if appropriate */
bool MapViewShadingLayerPrivate::handleMouseButtonRelease( QMouseEvent *e )
{
	if(m_bInputEnabled)
		return false;

	//m_selIdx = QModelIndex();
	return false;
}

/** Select route instructions points, start dragging trip points */
bool MapViewShadingLayerPrivate::handleMouseButtonPress( QMouseEvent *e )
{
	if(m_bInputEnabled)
		return false;

	if ( e->button() == Qt::LeftButton ) 
	 return false;

	QModelIndex idxParent = m_treeviewModel->getIndexFromIndexId(m_indexID);
	int iRowCount = m_treeviewModel->rowCount(idxParent);
	if(iRowCount==0)return false;

	//clicke over poly line
	foreach( const ModelRegion &region, m_shadingRegion ) 
	{
        if ( region.region.contains( e->pos() )  ) 
		{
			if ( e->button() == Qt::RightButton && m_bGrid ) 
			{
				int flag=0;
				GIS::TreeViewItemType cType = (GIS::TreeViewItemType) qVariantValue<unsigned int>( m_selIdx.data( TreeViewModel::TypeRole ) );	
				if (cType == GIS::Shading )
				{
					QVariant qVar = m_treeviewModel->data(m_selIdx,TreeViewModel::InfoRole);
					flag = qVar.toInt();
					flag <<=8;//grid id in high byte
				}
				QPoint pt = e->pos();
				flag+=3;
				emit q->signalNotifyMouseAction(pt,flag,q->m_InterfaceID);//right button
				showContextMenu( e->pos() );
				return true;
			}
		}
	}

	return false;
}

    /** Dragging trip points, route polygon hovering */
bool MapViewShadingLayerPrivate::handleMouseMove( QMouseEvent *e )
{
	if(m_bInputEnabled)
		return false;

	qreal lon( 0.0 ), lat( 0.0 );
	m_bGrid=false;
    if ( m_marbleWidget->geoCoordinates( e->pos().x(), e->pos().y(),
                                         lon, lat, GeoDataCoordinates::Radian ) ) 
	{
		if ( isInfoPoint( e->pos()) )
		{
			//m_marbleWidget->setCursor( Qt::ArrowCursor );
			//return true;
			m_bGrid=true;
		}

	}
	 
	return false;
}

    /** Escape to stop selecting points */
bool MapViewShadingLayerPrivate::handleKeyEvent( QKeyEvent *e )
{
	if(m_bInputEnabled)
		return false;

	 return false;
}

bool MapViewShadingLayerPrivate::isInfoPoint( const QPoint &point )
{
	foreach( const ModelRegion &region, m_shadingRegion ) 
	{
		if ( region.region.contains( point )  )
		{
			//m_infoID = region.index.internalId();
			m_selIdx = region.index;
			return true;
		}
	}

	m_selIdx = QModelIndex();
    return false;
}

MapViewShadingLayer::MapViewShadingLayer( MarbleWidget *widget, QWidget *parent, qint64 index,long interfaceID ) :
        QObject( parent ), d( new MapViewShadingLayerPrivate( this, widget,index ) )
{
    widget->installEventFilter( this );

	connect( widget, SIGNAL( visibleLatLonAltBoxChanged( GeoDataLatLonAltBox ) ),
            this, SLOT( setViewportChanged() ) );

	m_InterfaceID = interfaceID; 
	m_zValue=1.0;
	m_cmID=-1;
}

MapViewShadingLayer::~MapViewShadingLayer()
{
	d->m_qmImage.clear();
    delete d;
}

QStringList MapViewShadingLayer::renderPosition() const
{
    //return QStringList() << "HOVERS_ABOVE_SURFACE";
	return QStringList() << "ALWAYS_ON_TOP";
	//return QStringList() << "SURFACE";
}

bool MapViewShadingLayer::render( GeoPainter *painter, ViewportParams *viewport,
                           const QString& renderPos, GeoSceneLayer *layer )
{
    if ( d->m_marbleWidget->mapTheme()->head()->target() != "earth" ) {
        return false;
    }

    Q_UNUSED( viewport )
    Q_UNUSED( renderPos )
    Q_UNUSED( layer )

    painter->save();
    painter->autoMapQuality();

    

    if ( d->m_treeviewModel) {
        d->renderLayer( painter );
    }

	

    painter->restore();
	// d->m_viewportChanged = false;
    return true;
}

bool MapViewShadingLayer::eventFilter( QObject *obj, QEvent *event )
{
    Q_UNUSED( obj )

    if ( event->type() == QEvent::KeyPress ) {
        QKeyEvent *e = static_cast<QKeyEvent*>( event );
        return d->handleKeyEvent( e );
    }

    if ( event->type() == QEvent::MouseButtonPress ) {
        QMouseEvent *e = static_cast<QMouseEvent*>( event );
        return d->handleMouseButtonPress( e );
    }

    if ( event->type() == QEvent::MouseButtonRelease ) {
        QMouseEvent *e = static_cast<QMouseEvent*>( event );
        return d->handleMouseButtonRelease( e );
    }

    if ( event->type() == QEvent::MouseMove ) {
        QMouseEvent *e = static_cast<QMouseEvent*>( event );
        return d->handleMouseMove( e );
    }

    return false;
}

void MapViewShadingLayer::setModel ( TreeViewModel *model )
{
    d->m_treeviewModel = model;
	setViewportChanged();
}

void MapViewShadingLayer::synchronizeWith( QItemSelectionModel *selection )
{
    d->m_selectionModel = selection;
//    d->m_proxyModel = model;
}


void MapViewShadingLayer::setLayerDirty( bool dirty )
{
    d->m_layerDirty = dirty;

    /** @todo: The full repaint can be avoided. The route however has changed
      * and the exact bounding box needs to be recalculated before doing
      * a partly repaint, otherwise we might end up repainting only parts of the route
      */
    // d->m_marbleWidget->repaint( d->m_routeRegion );
    d->m_marbleWidget->update();
}


void MapViewShadingLayer::setViewportChanged()
{
    d->m_viewportChanged = true;
}

void MapViewShadingLayer::slotLayerContextMenu(QAction* pAction)
{
	QModelIndex idxParent = d->m_treeviewModel->getIndexFromIndexId(d->m_indexID);
	int iRowCount = d->m_treeviewModel->rowCount(idxParent);
	if(iRowCount==0)return;

	if(d!=NULL && pAction!=NULL)
	{
		if(d->m_pActionGroup!=NULL)
		{
			QAction* pMenuAction=NULL;
			QVariant qVar;
			int iVal=0;
			QList<QAction*> pList = d->m_pActionGroup->actions();
			int i=pList.size();
			for(int x=0;x<i;x++)
			{
				pMenuAction = pList.at(x);
				if(pMenuAction==pAction)
				{
					qVar = pAction->data();
					iVal = qVar.toInt();
					m_cmID=-1;
					if(d->m_selIdx.isValid())
						emit contextMenuSelected(d->m_selIdx,getInterfaceID(),iVal );
				}
			}
		}
	}

}

void MapViewShadingLayer::setContextMenu(QString qsMenu)
{
	if(qsMenu.isEmpty() || d==NULL)
		return;

	//first delete old menu entries
	d->delContextMenu();

	QStringList qsl;
	QString qsTag;
	QString qsVal;
	int iVal=0;
	int id=0;
	do{
		qsTag = qsMenu.section(',',id,id+1);
		if(!qsTag.isEmpty())
		{
			qsVal = qsTag.section(',',1,1);
			iVal = qsVal.toInt();
			qsVal = qsTag.section(',',0,0);
			d->addContextMenu(qsVal,iVal);
		}
		id+=2;
	}while(!qsTag.isEmpty());


}

void MapViewShadingLayer::updateImageList (long id, QImage *img )
{
	if(img!=NULL)
	{
		//QImage qImg = img->copy();
		d->m_qmImage.insert(id,*img);
	}
	else
		d->m_qmImage.remove(id);
}

void MapViewShadingLayer::setDimmerValue(int dValue)
{
	if ( d!=NULL)
		d->m_iDimmer=dValue;
}

void MapViewShadingLayer::setInputEnabled(bool val)
{
	if(d!=NULL)
		d->m_bInputEnabled = val;
}

void MapViewShadingLayer::setEdit(bool edit)
{
	if ( d!=NULL)
		d->m_bEdit=edit;
}

bool MapViewShadingLayer::getEdit()
{
	if ( d!=NULL)
		return d->m_bEdit;
	else return false;
}
	
void MapViewShadingLayer::setDelete(bool del)
{
	if ( d!=NULL)
		d->m_bDelete=del;
}

bool MapViewShadingLayer::getDelete()
{
	if ( d!=NULL)
		return d->m_bDelete;
	else return false;
}

void MapViewShadingLayer::setObjectEdit(bool move)
{
	if ( d!=NULL)//also set by O_EDIT which affects both
	{
		d->m_bMoveObj=move;
		d->m_bDeleteObj=move;
	}
}
	
void MapViewShadingLayer::setObjectMove(bool move)
{
	if ( d!=NULL)//also set by O_EDIT which affects both
	{
		d->m_bMoveObj=move;
	}
}
	
void MapViewShadingLayer::setObjectDelete(bool del)
{
	if ( d!=NULL)
		d->m_bDeleteObj=del;
}

bool MapViewShadingLayer::getObjectDelete()
{
	if ( d!=NULL)
		return d->m_bDeleteObj;
	else return false;
}

// Overriding LayerInterface to paint on top of the route
qreal MapViewShadingLayer::zValue() const
{
	return 2.0; //without, this layer is not visible (but render is called)
}

void MapViewShadingLayer::export2kml(QString& filename)
{
	if(d!=NULL)
		d->export2kml(filename);

}

void MapViewShadingLayer::exportOverlay2kml(GeoDataDocument* document,GeoDataFolder* folder)
{
	if(d!=NULL)
		d->exportOverlay2kml(document,folder);

}

void MapViewShadingLayer::slotDeleteTreeItem(int type,long id)
{
	if(type==173)//Shape
	{
		qint64 qID=(qint64)id;
		d->m_shadingRegion.clear();
	//	if(qID>0)
		//	d->removePlacemark(qID);
	}
}

} // namespace Marble

#include "moc_MapViewShadingLayer.cpp"
