/******************************************************************************************************************
 *	@file			MapViewShadingLayer.h
 *	@copyright		2013 Rohde&Schwarz, Munich
 *	@version		$Header: MapViewShadingLayer.h    
 *	5 Novemder 2011 9:49:35  enzensbe $
 *
 *	@author			Konrad Enzensberger		e.konrad@mpegcode.com
 *	@responsible	KOnrad Enzensberger
 * 
 *	@language		C++ ANSI V3
 *
 *	@description 	This file defines the MapView shading layer.
 *					
 *					
 *****************************************************************************************************************/

#ifndef MARBLE_MAPVIEWSHADINGLAYER_H
#define MARBLE_MAPVIEWSHADINGLAYER_H

#include "GeoDataCoordinates.h"
#include "LayerInterface.h"

#include <QtCore/QModelIndex>
#include <QtGui/QItemSelection>
#include <QtGui/QMenu>

#include "Enum.h"
using namespace GIS;

namespace Marble
{

class MarbleWidget;
class MapViewShadingLayerPrivate;
class TreeViewModel;
class GeoDataDocument;
class GeoDataFolder;
/**
  * @brief A shading layer that serves as a view off image data
  */
class MapViewShadingLayer: public QObject, public LayerInterface
{
    Q_OBJECT

public:
    /**
      * @brief Constructor
      * @param widget The marble widget used for geopos <-> screenpos transformations
      *   and repainting of (small) areas. Must not be null
      * @param parent Optional parent widget
      */
    explicit MapViewShadingLayer( MarbleWidget *widget, QWidget *parent = 0, qint64 index=0,long interfaceID=0);

    /** Destructor */
    ~MapViewShadingLayer();

    /** Reimplemented from LayerInterface. We'll hover above the surface */
    QStringList renderPosition() const;

    /** Reimplemented from LayerInterface. Paints route items and placemarks */
    bool render( GeoPainter *painter, ViewportParams *viewport,
                 const QString &renderPos = "NONE", GeoSceneLayer *layer = 0 );


    /**
      * The route is dirty (needs an update) and should be painted to indicate that
      */
    void setLayerDirty( bool dirty );

	 /**
      * get the Interface ID for TcpIp writing
      */
	long getInterfaceID(void){return m_InterfaceID;}

	 /**
      * Set the proxy model another QAbstractItemView uses that should share
      * its selection model with us. Needed because this class uses an unfiltered
      * model which has different indices than a filtered one.
      */
    void synchronizeWith( QItemSelectionModel *selection );

	 /**
      * Set the tree model to use. Implicitly removes the placemark model.
      */
    void setModel ( TreeViewModel *model );

	/**
      * Add/Remove new image.
      */
    void updateImageList (long id, QImage *img );

	/**
      * input enabled flag - if true, reject mouse move over object
      */
	void setInputEnabled(bool val);

	/**
      * set the layer editable 
      */
	void setEdit(bool edit);
	/**
      * get the layer editable flag
      */
	bool getEdit(void);
	/**
      * set the layer deleteable 
      */
	void setDelete(bool del);
	/**
      * get the layer deleteable flag
      */
	bool getDelete(void);
	/**
      * set the layer objects delete able 
      */
	void setObjectDelete(bool del);
	/**
      * get the layer objects delete able flag 
      */
	bool getObjectDelete(void);
	/**
      * set the layer objects moveable 
      */
	void setObjectMove(bool move);
	/**
      * set the layer objects moveable/deleteable 
      */
	void setObjectEdit(bool move);

	/**
      * set the layer context menu 
      */
	void setContextMenu(QString qsMenu);

	/**
      * set dimmer value to change text color
      */
	void setDimmerValue(int dValue);

	// Overriding LayerInterface to paint on top of other layers
    virtual qreal zValue() const;

	/**
      * export layer objects to kml
      */
	void export2kml(QString&);

	/**
      * export overlay objects to kml
      */
	void exportOverlay2kml(GeoDataDocument* document,GeoDataFolder* folder);


Q_SIGNALS:
   

    /**
      * The image region was changed permanently (no dragging),
      * new route instructions should be downloaded
      */
    void layerDirty();

	 /**
      * Selection of an object context menu by the user
      */
	void contextMenuSelected( const QModelIndex &index,long intID, int& iVal );

	/**
	  * send mouse pos 
	*/
	void signalNotifyMouseAction(QPoint&,int,long intID);

protected:
    /** Overriding QWidget, used to make the layer interactive */
    bool eventFilter( QObject *obj, QEvent *event );

public Q_SLOTS:
	/**
      * free selection index if item is deleted by tree
      */
	void slotDeleteTreeItem(int,long);

private Q_SLOTS:

	 /**
      * The viewport has changed, recalculate positions accordingly
      */
    void setViewportChanged();

	/**
      * send layer context menu id to TcpIp
	  * called from: 
	  * - layer context menu
	  * - multi select context menu 
      */
	void slotLayerContextMenu(QAction*);

private:
    MapViewShadingLayerPrivate *const d;
    friend class MapViewShadingLayerPrivate;

	long	m_InterfaceID;
	qreal	m_zValue;
	int		m_cmID;

};

} // namespace Marble

#endif
