/******************************************************************************************************************
 *	@file			TreeViewWidget.h
 *	@copyright		2010 Rohde&Schwarz, Munich
 *	@version		$Header: TreeViewWidget.h     
 *	5 November 2010 16:49:35  enzensbe $
 *
 *	@author			Konrad Enzensberger		e.konrad@mpegcode.com
 *	@responsible	KOnrad Enzensberger
 * 
 *	@language		C++ ANSI V3
 *
 *	@description 	This file defines the MapView TreeViewWidget.
 *					
 *					
 *****************************************************************************************************************/

#ifndef MARBLE_TREEVIEWWIDGET_H
#define MARBLE_TREEVIEWWIDGET_H

#include "GeoDataCoordinates.h"
#include "GeoDataLineString.h"
//#include "RoutingManager.h"

#include <QModelIndex>
#include <QWidget>
#include <QItemSelection>

class QTimer;

//#define PI		3.14159265358979323846

namespace Ui
{
    class LayerProperties;
	class ObjectProperties;
}

namespace Marble
{

class MarbleWidget;
//class RoutingInputWidget;
class TreeViewWidgetPrivate;
class MvPenStyleComboBox;
class FileManager;
class GeoDataDocument;
class GeoDataContainer;
class GeoDataPlacemark;

/**
  * A widget consisting of input fields for places / routing destinations,
  * a list view showing routing instructions and a interactive paint layer
  * showing placemarks and the route
  */
class MARBLE_EXPORT TreeViewWidget : public QWidget
{
    Q_OBJECT
 
public:
    /**
      * @brief Constructor
      * @param marbleWidget The marble widget used to register an event handler
      * on. Must not be null.
      * @param parent Optional parent widget
      */
    explicit TreeViewWidget(QWidget *parent );

    /** Destructor */
    ~TreeViewWidget();

	/**
     * @brief Add a MarbleWidget to be controlled by this widget.
     * @param widget  the MarbleWidget to be added.
     *
     *  Add a MarbleWidget so that it will be controlled by this
     *  widget.  This call create all the connections of signals and
     *  slots that are necessary to have full control of the
     *  MarbleWidget, and thereby frees the programmer from doing it.
     */
    void addMarbleWidget( MarbleWidget *marbleWidget);

    /**
      * Toggle offline mode. In offline mode, new entries can not be set
      */
    void setWorkOffline( bool offline );

	/**
	  * set bearing points
	  */
	void setBeamPointCount(int iBeamCount);
	
	/**
	  * experimental
	  */
	void resetEventFilter(bool set);

	/**
	  * export layer to kml
	  */
	void exportLayer(QString &);

	/**
	  * enable pan map to focus object
	  */
	void enablePanMapToObject(bool);

	/**
	  * drag&drop icons
	  */
	void addDropItem(QPixmap&, QPoint&, QString&);

	/**
	  * set dimmer value to layers
	  */
	void setDimmerShading(int value);

	/**
	  * call coordinate format widget
	  */
	void coordinateFormat(qreal lon,qreal lat);

	/**enz
     * @brief  receive gpx placemark 
     * @param  longitude - latitude - azimuth - name
     */
    void gpxPlacemark(long id,qreal lon,qreal lat,qreal azimuth, QString&);

	/**
	  *add GeoDataDocumnent contents
	  */
	void addGeoDataDocument(GeoDataDocument*);

	/**
	  *add GeoDataDocumnent symbole contents
	  */
	void addGeoDataDocumentSymbole(GeoDataDocument*);

	/**
	  *add map layer from GeoDataDocumnent
	  */
	long addMapLayerFromDocument(GeoDataDocument* doc,int type,QString& add=QString());

	/**
	  *check svg object
	  */
	bool checkSvgObjectKey(QString&);

	/**
	  * convert pixel pos to geo coordinates - check if pixel position within MapViewMapLayer/GroundOverlay
	  */
	void PixelToCoord(int xpos,int ypos, double& lon,double& lat);

	/**
	  * convert geo coordinates to pixel pos - check if pixel position within MapViewMapLayer/GroundOverlay
	  */
	void CoordToPixel(int& xpos,int& ypos, double lon,double lat);

	/**
	  * List of all avaliable layers
	  */
	bool mapLayerList(QStringList&);

	/**
	  * set layer zvalue
	  */
	bool setLayerZorder(QString& name,int flag,qint64 idx=0);

	/**
	  * add/remove document according zvalue
	  */
	void updateDocumentListing(long id,int flag);

	/**
      * Set the view context to determine whether the map is used interactively
      */
    void setViewContext( ViewContext viewContext );

	/**
      * get current loaded MapInfo Geoset
      */
	void geosetFilename(QString& gst);

	/**
      * export to kml - by embedded interface
      */
	void exportToKml(long id,QString& filename);

	/**
      * export a location xml
      */
	void exportToLocationMap(QString& filename);

	/**
      * enable/disable object selection
      */
	void disableSelection(bool flag,bool temp=false);

private:
	/** Add map layers to tree **/
	void addMapToTree();
	/** Remove map layers from tree **/
	void removeMapFromTree();
	/** Remove layer from marble **/
	void removeLayer(qint64 objId);
	/** delete objects from marble **/
	void deleteObjects(QStringList&,long id,int type);
	/** delete overlay from marble **/
	void deleteOverlay(long id);
	/** delete layer from marble **/
	void deleteLayer(long id);
	/** delete group from marble **/
	void deleteGroup(long id);
	/** delete shape from marble **/
	void deleteShape(long id);
	/** check if focus object is within map area 
	*   called for moved objects **/
	bool centerFocusObject(qreal lon, qreal lat);
	/** initialize the layer properties widget */
	void initLayerPropertiesWidget(int init);
	/** initialize the object properties widget */
	void initObjectPropertiesWidget(int init);

	//kml
	void savePlacemarks(const GeoDataContainer *container);
	//add GeoDataDocument contents to tree
	long createKmlPlacemarks( GeoDataContainer *container,long id=0 );
	long createGpxPlacemarks( GeoDataContainer *container,long id=0 );

	//tactical symbole /overlaylayer
	long createKmlTsPlacemarks( GeoDataContainer *container,long id=0 );

	/** recalculate plot points if group moved */
	bool calcPlotCoords(qint64 index,const GeoDataCoordinates& src,const GeoDataCoordinates& dest);

	//3D View
	void sendObjectsTo3D(bool send);

	//Romes
	void intersectionObjects(long layerID, long objectID,QPoint& pt,long intID );
	bool detectBTSflag( const QModelIndex &index,QPoint& mousePT,bool omni,bool fixed,int& flag);
	bool verifyGeosetLayer(int flag,qint64 qID,QString& name,qint64 idx);

//protected:
//    bool eventFilter(QObject* watched, QEvent* e);

private Q_SLOTS:

    /** User activated a route instruction element in the list view, synchronize paint layer */
    void activateItem ( const QModelIndex &index );

	 /** User deleted an object */
    void itemToDelete ( const QModelIndex &index,long intID );

    /** Placemark search is finished in an input field, switch to its placemark model */
    //void handleSearchResult( RoutingInputWidget *widget );

    /** Switch to the placemark model of an input field */
    //void centerOnInputWidget( RoutingInputWidget *widget );

    /** A placemark was selected in the map, synchronize tree view */
    void activatePlacemark( const QModelIndex &index,long intID,bool sel );

	/** A placemark was selected in the map, synchronize tree view */
    void activateMultiPlacemark( const QModelIndex &index,long intID,bool sel );

	/** A placemark was double clicked in the map, synchronize tree view */
	void dblClickPlacemark( const QModelIndex &index,long intID,QPoint& pt,bool sel );

	/** A polygone was selected in the map, synchronize tree view */
    void activatePolygone( const QModelIndex &index,long intID,bool sel );

	/** A polygone was double clicked in the map, synchronize tree view */
    void dblClickPolygone( const QModelIndex &index,long intID,QPoint& pt,bool sel );

	 /** A placemark was moved in the map, synchronize tree view */
	void retrieveMovedPlacemark( const QModelIndex &index,long intID );

	/** A placemark was moved in the map, synchronize tree view */
	void retrieveMovedPolygone( const QModelIndex &index,long intID );

	/** An object context menu was selected in the map, synchronize tree view */
	void retrieveContextMenu( const QModelIndex &index ,long intID, int& iVal );

	/** mouse pos hovered an object */
	void hoveredObject( const QModelIndex &index,QPoint& pos,long intID );

	/** A multi placemarks are selected on map, send list */
	void activateMultiSelection( const QMap<int,QModelIndex> &idxMap,long intID,bool sel );

    /** Insert another input field at the given position */
    //void insertInputWidget( int index );

    /** Add another input field at the end */
    //void addInputWidget();

    /** Remove an existing input field */
    //void removeInputWidget( RoutingInputWidget *widget );

    /** Remove an existing input field */
    //void removeInputWidget( int index );

    /** Route state changed */
   // void updateRouteState( RoutingManager::State state, RouteSkeleton *route );

    /** An input field requests a position input from the map */
    //void requestMapPosition( RoutingInputWidget *widget, bool enabled );

    /** Position in the map selected by the user after a previous slotMapInputRequested */
    void retrieveSelectedPoint( const GeoDataCoordinates &coordinates );

    /** Update the text of the Search / GetDirections button */
    //void adjustSearchButton();

    /** The user canceled point selection from the map */
    void pointSelectionCanceled();

    /** Show or hide routing options */
    //void toggleOptionsVisibility();

    /** Export route to a file */
    //void exportRoute();

    /** Progress animation update */
    //void updateProgress();

	/** show/hide layer */
	void showSelectedItem ( const QModelIndex & index ) ;
	/** zoom to selected tree item */
	void zoomSelectedItem ( const QModelIndex & index ) ;

	/** tree context menu */
	void contextTreeMenu(const QPoint &);
	void deleteTreeObject();
	void moveTreeItemUp();
	void moveTreeItemDown();
	void moveTreeItemToTop();
	void moveTreeItemToBott();
	void setTreeItemProperties();
	void setTreeItemObjectProperties();
	void exportObjects();

	/** layer properties slots */
	void chooseLayerLineColor();
	void chooseLayerFillColor();
	void chooseLayerFontColor();
	void synchronizeTransbox(int);
	void synchronizeWidthbox(int);

	/** object properties slots */
	void chooseObjectLineColor();
	void chooseObjectFillColor();
	void synchronizeObjTransbox(int);
	void synchronizeObjWidthbox(int);

public Q_SLOTS:
    void setCheckedProperty( const QString& name, bool checked );
	void selectTheme( const QString & );

	//signals from TcpIp interface
	void slotAddObject(QStringList,long id,int type,long& objectID);
	void slotAddImageObject(QString&,long id,int type,long xoffs, long yoffs, long align, long idx, long& objectID,bool bMulti);
	void slotAddPlotObject(QStringList&,long id,int type,double radius,double longitude,double latitude,long flags,GeoDataLineString&,long& objectID);
	void slotMoveObject(long id,double lon, double lat);
	void slotAddAzimuthLine(long id,int type,long azimuth,double longitude,double latitude,GeoDataLineString&,long& objectID,bool bMulti);
	void slotAddRect(long id,int type,double left, double top,double right,double bottom,long& objectID);
	void slotSetObjectProperties(QString&,long id,int type);
	void slotDeleteObject(long,int);
	void slotAddPoly(long id,int type,long lCol,GeoDataLineString&, long& objectID);
	void slotAddPolygon(long id,int type,long lCol,GeoDataLineString&, long& objectID);
	void slotAddColorPolygon(long id,QString& qs,long fCol,long lCol,int width,GeoDataLineString& gdl, long& objectID);
	void slotZoomToObject(long,int);
	void slotAddDescription(QString&,long id,long align,long xoff,long yoff,long& objectID);
	void slotAddShadingParameter(long& id,int iGridX,int iGridY,qreal left,qreal top,qreal right,qreal bottom, long& objectID);
	void slotAddShadingData(long idObj,long lID,long lCount,void* pData, long& objectID);
	void slotLayerObjects(int iType,long id,long lCount,QString& qs,long& ret);
	void slotAddArrow(long id,double longitude,double latitude,long lCol,int dirc1,int dirc2,GeoDataLineString&, long& objectID);
	void slotAddHyperbola(long id,qint64 time,double lon1, double lat1,double lon2,double lat2,long& objectID);

	void slotAddSvgObject(long id, QString& key,double lon, double lat, bool file,bool fixed,bool omni,int bts, QPixmap* pPix,long& objectID);
	void slotAddHotSpot(long id, QString& key,double lon, double lat, bool set, QPixmap* pPix,long& objectID);
	void slotAddPixmapObject(long id, QString& key,double lon, double lat,int xoffs,int yoffs,int alignment, QPixmap* pPix,long& objectID);
	void slotMoveAllObjects(long id,int x,int y,int& count);

	/** new colored area */
	void slotAddColoredAreaData(long idObj,long lID,long lCount,void* pData, long& objectID);

	//special delete for AddObjects
	void slotDeleteObjectId(long,int,int&,long&);

	//taktical situation symbols
	void slotAddTsSymbol(long id,long symID,long linecol,long fillcol,long width,long& objectID);

	//not from TcpIp
	void slotAddEllipse(QString&,long id,int type,double radius,double longitude,double latitude, double minor, double major, long& objectID);

	void slotUpdateMap();
	void slotUpdatePoly(long id,int type,long lCol,GeoDataLineString& gdl, long& objectID);

	//sent by GeoObjectsPlugin if selected Overlay/Layer is there a member
	void slotSituationOverlay(long ovlID);

	//called by focus object timer
	void slotCenterMap();

	//elevation detection
	void addDetectedElevationResult( const GeoDataCoordinates pos,const qreal val  );

	//kml/gpx file parser result
	void addParsingFileResult( GeoDataDocument* document);

	/**
	  * mouse position in geo coordinates
	  */
	void slotMouseGeoPos(const qreal lon, const qreal lat);

	/** 
		mouse click event
	*/
	void notifyMouseAction(QPoint& pos,int flag,long intID=0);

	/** 
		mouse click event over maplayer image
	*/
	void notifyImageMouseAction(QPoint& pos,QPoint& posImg,int flag,long intID);

	/**
     * @brief Set a list/set of placemark names for the search widget.
     * @param locations  the QAbstractitemModel containing the placemarks
     *
     * This function is called to display a potentially large number
     * of placemark names in a widget and let the user search between
     * them.
     * @see centerOn
     */
    void setLocations( QVector<GeoDataPlacemark*> locations );

	/**
	* This function is called to display a potentially large number
    * of placemark names in a widget and let the user search between
    * them.
	*/
	void slot3Denable(bool);

	/**
	* This function is called to add a map overlay.
	*/
	void slotAddMapObject(int type,GeoDataDocument* pMap,long& objectID);

	/**
	* This function is called to multi select object by rubber band
	*/
	void selectRegion( const QList<double>& );

	/**
	* This function is called by tree header selection
	*/
	void customSortByColumn(int column);

	/**
	*	update new ground overlay image 
	*/
	void slotUpdateFeature(long id,qreal left,qreal top,qreal right,qreal bottom);

	/**
	*	set tree node visible/invisible 
	*/
	void slotObjectVisible(long id,bool state);

	/**
	*	set tree node visible/invisible 
	*/
	void slotLayerVisible(long id,QString& name,bool state);

	/**
	*	check object intersection
	*/
	void slotObjectIntersection(long id,double longitude,double latitude,long& visible);

	/**
	*	tree item(s) selection changed
	*/
	void slotTreeSelectionChanged( const QItemSelection & selected, const QItemSelection & deselected );

Q_SIGNALS:

/*
    void toggledLocations( bool );
    void toggledCities( bool );
    void toggledTerrain( bool );
    void toggledBorders( bool );
    void toggledWaterBodies( bool );
    void toggledOtherPlaces( bool );
    void toggledIceLayer( bool );
    void toggledGrid( bool );
    void toggledRelief( bool );
    void toggledCompass( bool );
    void toggledScaleBar( bool );
*/

    void toggledShowProperty( QString, bool );
	void signalSendEvent(long type,long size,void* pData,long intID);
	//for situation layers
	void signalSelectedItem(long overlayID,long layerID,int id);
	//for create items via interface command
	void signalAddItem(int iType,long overlayID,long layerID,long groupID,long itemID);
	//to remove from QHash
	void signalDeleteObject(int type,long id);
	//send elevation value to app
	void signalElevationValue(qreal);
	//send current position to app, called by rmb button
	void signalCoordinateFormat(qreal,qreal,long);

	//send measure line to elevation profile plugin
	void signalUpdateElevationProfile(GeoDataLineString&);
	//send Gpx placemark to GeoObjects - calculates the bearing
	void signalGpxPlacemark(long id,long overlay,long layer,qreal lon,qreal lat,qreal azimuth, QString&);

	/*** 3D signals *********************************************/
	void signal3dAddImageObject(long id,QString& img,double lon,double lat);
	void signal3dAddMarkerObject(long id,QString& img,int type,double lon,double lat);
	void signal3dMoveObject(long id,double lon,double lat);
	void signal3dAddAzimuth(long id,double lon,double lat,long azi,GeoDataLineString& gdl);
	void signal3dAddPlot(long id,QStringList& img,double lon,double lat,double radius,GeoDataLineString& gdl);
	void signal3dAddPlotEx(long id,QStringList& img,double lon,double lat,GeoDataLineString& gdl);
	void signal3dAddGeoObject(long id,int type,double lon,double lat,long fillcol,long linecol,GeoDataLineString& gdl);
	void signal3dAddText(long id,QString& qs,double lon,double lat,long align,long xoff,long yoff);

	/*** MapInfo GeoSet *****************************************/
	void signalMapLayerItem(int,long,long,const QString&);

	/** grid toogeled signal **/
	void signalSetGrid(bool);
	/** grayscaled toogeled signal **/
	void signalSetGray(bool);
	/** grayscaled toogeled signal **/
	void signalSetElevation(bool);

	/** sent to plugins who created a kml import **/
	void signalObjectImport(long overlayID,long layerID, const QString& filename);

private:
    TreeViewWidgetPrivate *const d;
	QTimer*			m_pUpdateTimer;
	QTimer*			m_pFocusTimer;
	QRegion			m_dirtyRG;
	qint64			m_selectedID;
	qint64			m_focusID;
	qint64			m_sitOverlayID;
	qint64			m_vecOverlayID;
	qint64			m_propOverlayID;
	qint64			m_vecLayerID;
	QHash<int,long>	m_layerObj;
	QHash<long,int>	m_overlayLegend;
	bool			m_bRedraw;
	bool			m_bFocusObject;
	int				m_iDimmer;

	//tactical symbol layer
	qint64			m_tsOverlayID;
	qint64			m_tsLayerID;
	qint64			m_tsGroupID;
	bool			m_bMOG;

	//custom maps
	qint64			m_esriCacheID;		//ESRI tiling cache map
	qint64			m_minfOverlayID;	//MapInfo raster/vector layer
	qint64			m_esriOverlayID;	//ESRI shape layer
	qint64			m_mapOverlayID;	//Map layer

	QString			m_qsGeoset;

	//srtm enabled ?
	bool			m_bElevation;
	qint64			m_lElevationID; 
	//coordinate grid enabled ?
	bool			m_bGrid;
	qint64			m_lGridMapID; 
	//grayscaled grid enabled ?
	bool			m_bGray;
	qint64			m_lGrayMapID; 

	//layer/object settings
	Ui::LayerProperties  *ui_lpWidget;
	Ui::ObjectProperties *ui_opWidget;
    QDialog *m_lpDialog;
	QDialog *m_opDialog;
	MvPenStyleComboBox* m_lsComboBox;
	MvPenStyleComboBox* m_osComboBox;
	QColor			m_colLine;
	QColor			m_colFill;
	QColor			m_colFont;
	int				m_iPropSet;
	int				m_iRepaintCount;

	QHash<qint64,GeoDataCoordinates*>	m_objectCache;

	//the overall legend rect
	QRect m_legendRect;
	int   m_iARGUS;

	//enable 3D
	bool			m_b3D;
};

} // namespace Marble

#endif
